------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use  Gtk;
with Gtk.Main;


with Ada.Io_Exceptions;
use  Ada.Io_Exceptions;

with  Gnat.Current_Exception;
use  Gnat.Current_Exception;


with Ada.Strings.Unbounded;
use  Ada.Strings.Unbounded;

with Unbounded_Strings;
use  Unbounded_Strings;

with Version;
use  Version;
with Config;
use  Config;

with Project;
use  Project;
with Systems;
use  Systems;
with Task_Set;
use Task_Set;
with Processor_Set;
use Processor_Set;
with Buffer_Set;
use Buffer_Set;
with Resource_Set;
use Resource_Set;
with Address_Space_Set;
use Address_Space_Set;

with Xml_Parsers;
use Xml_Parsers;
with Aadl_Parsers;
use Aadl_Parsers;

with Root_Window_Pkg;
use  Root_Window_Pkg;
with Graph_Window_Pkg;
use  Graph_Window_Pkg;
with My_Canvas;
use  My_Canvas;


with Gtk.Widget;
use  Gtk.Widget;
with Gdk.Font;
use  Gdk.Font;
with Gtk.Style;
use  Gtk.Style;

with Gnat.Command_Line;
use  Gnat.Command_Line;

with Gnat.Os_Lib;
use  Gnat.Os_Lib;

with Text_Io;
use  Text_Io;

with Config;
use Config;
with Translate;
use Translate;

with Translate.English_Labels;
with Translate.Francais_Labels;


procedure Cheddar is 



   procedure Usage is 
   begin
      Copyright("cheddar");
      Put_Line (
         "Cheddar is a free real time scheduling simulator.");
      Put_Line(
         "This program provides services to automatically check temporal constraints of real time tasks.");
      New_Line;
      Put_Line(
         "Check Cheddar home page for details :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : cheddar [switch] xml-project-file");
      Put_Line("   switch can be :");
      Put_Line("            -u get this help");
      Put_Line(
         "            -l select a language : ""fr"" for franais; ""en"" for english ");
      Put("               Default language is ");
      Put_Line(
         "            -a the file name given to cheddar contains an AADL description instead of an XML file ");
      if(Current_Language = English)
            then
         Put("English.");
      else
         Put("Franais.");
      end if;
      New_Line;
   end Usage;


   My_Style          : Gtk_Style;  
   Global_Font       : Gdk_Font;  
   Aadl_File         : Boolean;  
   Project_File_Name : Unbounded_String;  


begin

   -- New project
   --
   Aadl_File:=False;
   Project_File_Name:=To_Unbounded_String("");
   Do_New;



   -- Get arguments
   --
   loop
      case Getopt ("u l: a") is
         when Ascii.Nul =>
            exit;

         when 'a' =>
            Aadl_File:=True;

         when 'u' =>
            Usage;
            Os_Exit(0);

         when 'l' =>
            if (Parameter /= "fr") and
                  (Parameter /= "en")
                  then
               Put_Line("cheddar.adb : invalid language parameter");
               New_Line;
               Usage;
               Os_Exit(0);
            else
               if Parameter = "fr"
                     then
                  Current_Language:=Francais;
               else
                  Current_Language:=English;
               end if;
            end if;

         when others =>
            Usage;
            Os_Exit(0);
      end case;
   end loop;


   loop
      declare
         S : constant String := Get_Argument (Do_Expansion => True);  

      begin
         exit when S'Length = 0;
         Project_File_Name :=Project_File_Name & S;
      end;
   end loop;


   Copyright("cheddar");


   Translate.Initialize;
   if Current_Language=English then
      Translate.English_Labels.Initialize;
   else
      Translate.Francais_Labels.Initialize;
   end if;


   if(Project_File_Name /= Empty_String)
         then
      Is_New:=False;
      Is_Saved:=True;
      if (Aadl_File)
            then
         Aadl_Project_File_Name:=Project_File_Name;
         Aadl_Project_Directory:=Project_File_Name;
         Systems.Read_From_Aadl_File(Sys, Aadl_Project_File_Name);

         -- 
         -- The piece of code below allows us to call aadl2xml externally instead of 
         -- directly calling the package aadl_parser.ads
         --
         --         declare
         --            Ok                        : Boolean;  
         --            Spawn_Args                : String_List (1 .. 2);  
         --            Temp_File_Name            : aliased String       := "-o cheddar_temp_file.xml";  
         --            Aadl_File_Name_To_Perform : aliased String       := To_String
         --              (Aadl_Project_File_Name);  
         --         begin
         --            Spawn_Args(1) := Temp_File_Name'Unchecked_Access;
         --            Spawn_Args(2):=Aadl_File_Name_To_Perform'Unchecked_Access;
         --            Spawn("aadl2xml", Spawn_Args, Ok);
         --            if not Ok then
         --               Put_Line("marche pas");
         --            end if;
         --         end;

      else
         Xml_Project_File_Name:=Project_File_Name;
         Xml_Project_Directory:=Project_File_Name;
         Systems.Read_From_Xml_File(Sys, Xml_Project_File_Name);
      end if;
   end if;



   -- Initialize the graphical editor
   --
   Gtk.Main.Set_Locale;
   Gtk.Main.Init;

   Load (Global_Font,
      "-*-*-*-*-*-*-*-*-*-*-*-*-iso8859-1");
   if Global_Font = Null_Font
         then
      Put_Line("cheddar.adb : warning, can not load ISO 8859-1 font ");
   end if;

   Gtk_New(My_Style);
   Set_Font(My_Style, Global_Font);
   Set_Default_Style (My_Style);

   Gtk_New (Root_Window);
   Show_All(Root_Window);


   -- Graphical editor event loop
   --
   Gtk.Main.Main;



exception

   when Task_Set.Invalid_Parameter =>
      Put_Line ("cheddar.adb : invalid task argument ; " &
         Exception_Message);
   when Address_Space_Set.Invalid_Parameter =>
      Put_Line ("cheddar.adb : invalid addresss space argument ; " &
         Exception_Message);
   when Processor_Set.Invalid_Parameter =>
      Put_Line ("cheddar.adb : invalid processor argument ; " &
         Exception_Message);
   when Buffer_Set.Invalid_Parameter =>
      Put_Line ("cheddar.adb : invalid buffer argument ; " &
         Exception_Message);
   when Resource_Set.Invalid_Parameter =>
      Put_Line ("cheddar.adb : invalid resource argument ; " &
         Exception_Message);

   when Gnat.Command_Line.Invalid_Switch    =>
      begin
         Put_Line ("cheddar.adb : invalid Switch ; " & Full_Switch);
         Usage;
      end;
   when Gnat.Command_Line.Invalid_Parameter =>
      begin
         Put_Line ("cheddar.adb : missing parameter for switch ; " &
            Full_Switch);
         Usage;
      end;

   when Xml_Read_Error =>
      Put_Line(
         "cheddar.adb : Can not read project file ; Xml Fatal Error ; "
         & Exception_Message);
   when Aadl_Read_Error =>
      Put_Line(
         "cheddar.adb : Can not read project files ; AADL parsing Error ; "
         &  Exception_Message);

   when Ada.Io_Exceptions.Name_Error =>
      Put_Line("cheddar.adb : Can not open project files, Name_Error");
   when Ada.Io_Exceptions.Status_Error =>
      Put_Line("cheddar.adb : Can not open project files, Status_Error");
   when Ada.Io_Exceptions.Mode_Error   =>
      Put_Line("cheddar.adb : Can not open project files, Mode_Error");
   when Ada.Io_Exceptions.Use_Error     =>
      Put_Line("cheddar.adb : Can not open project files, Use_Error");
   when Ada.Io_Exceptions.Device_Error  =>
      Put_Line("cheddar.adb : Can not open project files, Device_Error");
   when Ada.Io_Exceptions.End_Error    =>
      Put_Line("cheddar.adb : Can not open project files, End_Error");
   when Ada.Io_Exceptions.Data_Error   =>
      Put_Line("cheddar.adb : Can not open project files, Data_Error");
   when Ada.Io_Exceptions.Layout_Error  =>
      Put_Line("cheddar.adb : Can not open project files, Layout_Error");

   when others =>
      Put_Line("This is an internal Cheddar bug ... sorry");
      Put_Line(
         "WARNING           : your project was saved in the file cheddar_bug.xml");
      Put_Line("Exception name    : " & Exception_Name);
      Put_Line("Exception message : " & Exception_Message);

      if Graph_Window/=null
            then
         Get_Coordinates(Graph_Window.Buffer, Sys);
      end if;
      Systems.Write_To_Xml_File(Sys, To_Unbounded_String(
            "cheddar_bug.xml"));

      Put_Line("Please, send a bug report to cheddar@univ-brest.fr");
      Put_Line(
         "Do not forget to join your XML Cheddar project files with your bug report");

end Cheddar;




