------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automatically check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;

with Gnat.Command_Line;
use Gnat.Command_Line;
with  Gnat.Os_Lib;
use Gnat.Os_Lib;

with Ada.Io_Exceptions;
use  Ada.Io_Exceptions;
with  Gnat.Current_Exception;
use  Gnat.Current_Exception;

with Systems;
use Systems;

with Parser;
use Parser;
with  Scheduler_Io;
use  Scheduler_Io;
with Text_Io;
use Text_Io;

with Version;
use Version;

with Config;
use Config;
with Translate;
use Translate;

with Expressions;
use Expressions;
use Expressions.Variables_Type_Package;


with Io_Tools;
use Io_Tools;


procedure Sc is 


   Sc_File,  
   File_Name : Unbounded_String;  




   procedure Usage is 
   begin
      Copyright("sc");
      Put_Line (
         "Sc is a compiler that checks if a user-defined code file is correct or not.");

      New_Line;
      Put_Line(
         "Check Cheddar home page for details :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : sc [switch] sc-file-to-check");
      Put_Line("   switch can be :");
      Put_Line("            -u get this help");
      Put_Line("            -v verbose mode ");
      New_Line;
   end Usage;


   -- Is the sc command should run in verbose mode ?
   --
   Verbose : Boolean := False;  


   -- The parser have to read a system description to
   -- set user defined parameters ...
   -- This is a "empty" dummy application description.
   --
   Dummy : System;  




begin

   -- Get arguments 
   --
   loop
      case Getopt ("u v") is
         when Ascii.Nul =>
            exit;

         when 'v' =>
            Verbose:=True;
         when 'u' =>
            Usage;
            Os_Exit(0);

         when others =>
            Usage;
            Os_Exit(0);
      end case;
   end loop;

   loop
      declare
         S : constant String := Get_Argument (Do_Expansion => True);  

      begin
         exit when S'Length = 0;
         File_Name := File_Name & S;
      end;
   end loop;


   Copyright("sc");

   if(File_Name = Empty_String)
         then
      Usage;
      Os_Exit(0);
   end if;


   -- Parse the file
   --
   if Verbose then
      Put("Parsing the file ");
      Put(To_String(File_Name) & " ...." );
   end if;
   New_Line;
   New_Line;
   New_Line;
   New_Line;


   Sc_File := Read_Sequential_File(File_Name);

   -- Start parser
   --
   Open_Input(Sc_File);
   Parser.Init_Parser(Dummy.Tasks);



   Parser.Yyparse;


   -- Display syntax tree and compiling information
   --
   if Verbose then
      Put(Root_Statement_Pointer);
      Put(Variables_Table);
   end if;




   New_Line;
   New_Line;
   New_Line;
   Put_Line("Syntax of the file " & To_String(File_Name) & " is ok ");
   New_Line;


exception
   when Expressions.Syntax_Error =>
      begin
         New_Line;
         New_Line;
         Put_Line(To_String(Lb_Syntax_Error(Current_Language) & Lb_Colon &
               To_Unbounded_String(Exception_Message)));

         New_Line;
      end;
   when Gnat.Command_Line.Invalid_Switch    =>
      begin
         Put_Line ("sc.adb : invalid Switch " & Full_Switch);
         Usage;
      end;
   when Gnat.Command_Line.Invalid_Parameter =>
      Put_Line ("sc.adb : missing parameter for switch " &
         Full_Switch);

   when Ada.Io_Exceptions.Name_Error =>
      Put_Line("sc.adb : Can not open project files, Name_Error");
   when Ada.Io_Exceptions.Status_Error =>
      Put_Line("sc.adb : Can not open project files, Status_Error");
   when Ada.Io_Exceptions.Mode_Error   =>
      Put_Line("sc.adb : Can not open project files, Mode_Error");
   when Ada.Io_Exceptions.Use_Error     =>
      Put_Line("sc.adb : Can not open project files, Use_Error");
   when Ada.Io_Exceptions.Device_Error  =>
      Put_Line("sc.adb : Can not open project files, Device_Error");
   when Ada.Io_Exceptions.End_Error    =>
      Put_Line("sc.adb : Can not open project files, End_Error");
   when Ada.Io_Exceptions.Data_Error   =>
      Put_Line("sc.adb : Can not open project files, Data_Error");
   when Ada.Io_Exceptions.Layout_Error  =>
      Put_Line("sc.adb : Can not open project files, Layout_Error");


   when others =>
      Put_Line("This is an internal sc bug ... sorry");
      Put_Line("Exception name : " & Exception_Name);
      Put_Line("Exception message : " & Exception_Message);
      Put_Line("Please, send a bug report to cheddar@univ-brest.fr");
      Put_Line(
         "Do not forget to join the XML/SC Cheddar files with your bug report");



end Sc;



