------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automatically check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Ada.Io_Exceptions;
use  Ada.Io_Exceptions;
with  Gnat.Current_Exception;
use  Gnat.Current_Exception;

with Unbounded_Strings;
use  Unbounded_Strings;

with Gnat.Command_Line;
use Gnat.Command_Line;

with  Gnat.Os_Lib;
use Gnat.Os_Lib;

with Text_Io;
use Text_Io;

with Version;
use Version;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Systems;
use Systems;
with Xml_Parsers;
use Xml_Parsers;

with Task_Set;
use Task_Set;
with Processor_Set;
use Processor_Set;
with Address_Space_Set;
use Address_Space_Set;
with Resource_Set;
use Resource_Set;
with Buffer_Set;
use Buffer_Set;


procedure xml2xml is 



   Sys       : System;  
   File_Name : Unbounded_String;  



   procedure Usage is 
   begin
      Copyright("xml2xml");
      Put_Line (
         "xml2xml allows you to check if a Cheddar project file is consistent or not.");

      New_Line;
      Put_Line(
         "Check Cheddar home page for details :  http://beru.univ-brest.fr/~singhoff/cheddar ");
      New_Line;
      New_Line;
      Put_Line("Usage : xml2xml xml-project-name");
      New_Line;
      New_Line;
   end Usage;




begin


   -- Get arguments 
   --
   loop
      case Getopt ("u") is
         when Ascii.Nul =>
            exit;

         when 'u' =>
            Usage;
            Os_Exit(0);

         when others =>
            Usage;
            Os_Exit(0);
      end case;
   end loop;

   loop
      declare
         S : constant String := Get_Argument (Do_Expansion => True);  

      begin
         exit when S'Length = 0;
         File_Name := File_Name & S;
      end;
   end loop;

   Copyright("xml2xml");

   if(File_Name = Empty_String)
         then
      Usage;
      Os_Exit(0);

   else
      Initialize(Sys);

      Read_From_Xml_File(Sys, File_Name);

      Put_Xml(Sys);

   end if;



exception
   when Gnat.Command_Line.Invalid_Switch    =>
      begin
         Put_Line ("xml2xml.adb : invalid Switch " & Full_Switch);
         Usage;
      end;
   when Gnat.Command_Line.Invalid_Parameter =>
      Put_Line ("xml2xml.adb : missing parameter for switch " &
         Full_Switch);

   when Xml_Read_Error =>
      Put_Line(
         "xml2xml.adb : Can not read project file ; Xml Fatal Error ; "
         & Exception_Message);

   when Task_Set.Invalid_Parameter =>
      Put_Line ("xml2xml.adb : invalid task argument ; " &
         Exception_Message);
   when Address_Space_Set.Invalid_Parameter =>
      Put_Line ("xml2xml.adb : invalid addresss space argument ; " &
         Exception_Message);
   when Processor_Set.Invalid_Parameter =>
      Put_Line ("xml2xml.adb : invalid processor argument ; " &
         Exception_Message);
   when Buffer_Set.Invalid_Parameter =>
      Put_Line ("xml2xml.adb : invalid buffer argument ; " &
         Exception_Message);
   when Resource_Set.Invalid_Parameter =>
      Put_Line ("xml2xml.adb : invalid resource argument ; " &
         Exception_Message);


   when Ada.Io_Exceptions.Name_Error =>
      Put_Line("xml2xml.adb : Can not open project files, Name_Error");
   when Ada.Io_Exceptions.Status_Error =>
      Put_Line("xml2xml.adb : Can not open project files, Status_Error");
   when Ada.Io_Exceptions.Mode_Error   =>
      Put_Line("xml2xml.adb : Can not open project files, Mode_Error");
   when Ada.Io_Exceptions.Use_Error     =>
      Put_Line("xml2xml.adb : Can not open project files, Use_Error");
   when Ada.Io_Exceptions.Device_Error  =>
      Put_Line("xml2xml.adb : Can not open project files, Device_Error");
   when Ada.Io_Exceptions.End_Error    =>
      Put_Line("xml2xml.adb : Can not open project files, End_Error");
   when Ada.Io_Exceptions.Data_Error   =>
      Put_Line("xml2xml.adb : Can not open project files, Data_Error");
   when Ada.Io_Exceptions.Layout_Error  =>
      Put_Line("xml2xml.adb : Can not open project files, Layout_Error");

   when others =>
      Put_Line("This is an internal xml2xml bug ... sorry");
      Put_Line("Exception name : " & Exception_Name);
      Put_Line("Exception message : " & Exception_Message);
      Put_Line("Please, send a bug report to cheddar@univ-brest.fr");
      Put_Line(
         "Do not forget to join the XML Cheddar project files with your bug report");





end  xml2xml;



