------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Ada.Finalization;

with Systems;
use Systems;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

package Aadl_Parsers is


   -- Print debug information to the consule if set to true
   --
   Aadl_Debug : Boolean := True;  


   -- Exception raised by clients of Xml_Parses when something goes
   -- wrong during XML parsing
   --
   Aadl_Read_Error : exception;  



   -- aadl parser
   --
   type Aadl_Project_Parser is new Ada.Finalization.Controlled with private; 


   -- Methods for all Cheddar XML parsers
   --
   procedure Initialize_Project_Parser (
         Handler : in out Aadl_Project_Parser ); 

   function Get_Parsed_System (
         Handler : in     Aadl_Project_Parser ) 
     return System; 

   procedure Initialize (
         Handler : in out Aadl_Project_Parser ); 

   procedure Parse (
         Handler   : in out Aadl_Project_Parser; 
         File_Name : in     Unbounded_String     ); 



private


   type Aadl_Project_Parser is new Ada.Finalization.Controlled with 
      record 
         -- At the end of the parser execution, the project
         -- is store in the objet below
         --
         Parsed_System : System;  
      end record; 






end Aadl_Parsers;



