------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Objects;
use Objects;


with Unchecked_Deallocation;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


package Address_Spaces is



   type Address_Space is new Generic_Object with 
      record 
         Cpu_Name : Unbounded_String;  

         Text_Memory_Size  : Natural := 0;  
         Stack_Memory_Size : Natural := 0;  
         Data_Memory_Size  : Natural := 0;  
         Heap_Memory_Size  : Natural := 0;  
      end record; 

   type Address_Space_Ptr is access all Address_Space'Class; 


   procedure Initialize (
         A_Addr : in out Address_Space ); 


   procedure Put_Name (
         A_Addr : in     Address_Space_Ptr ); 

   procedure Put (
         A_Addr : in     Address_Space_Ptr ); 

   procedure Put (
         A_Addr : in     Address_Space ); 

   function Copy (
         A_Addr : in     Address_Space_Ptr ) 
     return Address_Space_Ptr; 

   procedure Free is 
   new Unchecked_Deallocation
      (Address_Space'Class, Address_Space_Ptr);





end Address_Spaces;








