------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Config;
use Config;


with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Ada.Strings.Unbounded.Text_Io;
use Ada.Strings.Unbounded.Text_Io;

with Objects;
use Objects;


with Unchecked_Deallocation;
with Convert_Strings;
with Convert_Unbounded_Strings;
with Indexed_Tables;

with Ada.Numerics.Aux;
use Ada.Numerics.Aux;

with Queueing_System;
use Queueing_System;

package Buffers is


   type Role_type is 
         (No_Role,  
          Producer, 
          Consumer); 

   type Buffer_Role is 
      record 
         The_Role : Role_type;  
         Size     : Natural;  
         Time     : Natural;  
      end record; 


   procedure Put (
         B : in     Buffer_Role);
   procedure Initialize (
         B :    out Buffer_Role);


   procedure To_Role is 
   new Convert_Strings(Role_type, No_Role);
   procedure To_Role is 
   new Convert_Unbounded_Strings(Role_type, No_Role);



   package Buffer_Roles is new Indexed_Tables(Buffer_Role,
      Unbounded_String,
      Config.Max_Tasks, 0,
      Put, Initialize, Put);
   use Buffer_Roles;

   subtype Buffer_Roles_Table is Indexed_Table;
   subtype Buffer_Roles_Range is Indexed_Table_Range;

   --------------------------------------------------------

   type Buffer is new Generic_Object with
      record
         Cpu_Name : Unbounded_String := To_Unbounded_String ("");

         Address_Space_Name : Unbounded_String := To_Unbounded_String ("");

         Queueing_System_Type : Queueing_Systems_Type;

         Size  : Natural            := 0;
         Roles : Buffer_Roles_Table;
         X     : Natural            := 0;
         Y     : Natural            := 0;
      end record;

   type Buffer_Ptr is access all Buffer'Class;


   procedure Initialize (
         A_Buff : in out Buffer);

   procedure Put (
         My_Buff : in     Buffer);


   procedure Put_Name (
         A_Buff : in     Buffer_Ptr);

   procedure Put (
         A_Buff : in     Buffer_Ptr);


   function Copy (
         My_Buffer : in     Buffer_Ptr)
     return Buffer_Ptr;

   procedure Free is
   new Unchecked_Deallocation
      (Buffer'Class, Buffer_Ptr);



   --------------------------------------------------------


   function Is_Cons_Prod_Harmonic (
         My_Buff  : in     Buffer_Ptr;
         My_Tasks : in     Tasks_Set)
     return Boolean;


   Flow_Constraint_Not_Respected : exception;
   Task_Model_Error              : exception;


   procedure Buffer_Flow_Control (
         My_Buff   : in     Buffer_Ptr;
         My_Tasks  : in     Tasks_Set;
         Flow_Cons : in out Double;
         Flow_Prod : in out Double);




end Buffers;





