------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Statements;
use Statements;

with Expressions;
use Expressions;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;



with Unchecked_Deallocation;




package Event_Analyzers is


   type Event_Analyzer is tagged 
      record 
         -- Event Analyzer behavior in string format
         --
         String_Behavior : Unbounded_String;  

         -- file name which store the event Analyzer behavior 
         --
         Behavior_File_Name : Unbounded_String;  

         -- Statements for each scheduling
         -- section (pointers given by the parser)
         --
         Root_Statement_Pointer : Section_Pointers;  

         -- Variables table for the parsed parametric file
         -- (given by the parser)
         --
         Variables_Table : Variables_Table_Type;  


         -- Table of set statements given by the parser
         --
         Sets_Table : Sets_Table_Type;  
      end record; 

   type Event_Analyzer_Ptr is access all Event_Analyzer'Class; 


   procedure Put (
         My_Event_Analyzer : in     Event_Analyzer_Ptr ); 




   procedure Free is 
   new Unchecked_Deallocation
      (Event_Analyzer'Class, Event_Analyzer_Ptr);


   function Copy (
         My_Event_Analyzer : in     Event_Analyzer_Ptr ) 
     return Event_Analyzer_Ptr; 


end Event_Analyzers;











