------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Ada.Finalization;
with Unchecked_Deallocation;
with Convert_Strings;
with Convert_Unbounded_Strings;



package Objects  is




   function Is_A_Valid_Identifier (
         Id : Unbounded_String ) 
     return Boolean; 




   type Objects_Type is 
         (Buffer_Object_Type,        
          Processor_Object_Type,     
          Task_Object_Type,          
          Resource_Object_Type,      
          Address_Space_Object_Type, 
          Message_Object_Type,       
          Network_Object_Type); 


   procedure To_Object_Type is 
   new Convert_Strings(Objects_Type, Task_Object_Type);
   procedure To_Object_Type is 
   new Convert_Unbounded_Strings(Objects_Type, Task_Object_Type);


   --------------------------------------------------

   type Generic_Object is new Ada.Finalization.Controlled with 
      record 

         -- Object  name
         --
         Name : Unbounded_String := To_Unbounded_String ("");  

         -- Objet type (task, buffer, processor, ...)
         --
         Object_Type : Objects_Type;  
      end record; 


   type Object_Ptr is access all Generic_Object'Class; 


   procedure Put (
         My_Object : in     Generic_Object ); 

   procedure Put (
         My_Object : in     Object_Ptr ); 

   procedure Put_Name (
         My_Object : in     Object_Ptr ); 


   function Copy (
         My_Object : in     Object_Ptr ) 
     return Object_Ptr; 


   procedure Free is 
   new Unchecked_Deallocation
      (Generic_Object'Class, Object_Ptr);



end Objects;





