------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Package description :
--
--
--
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Text_Io;
use Text_Io;

with Ada.Strings.Unbounded.Text_Io;
use Ada.Strings.Unbounded.Text_Io;


with Ada.Exceptions;
use Ada.Exceptions;


with Config;
use Config;
with Translate;
use Translate;

with Time_Units;
use Time_Units;

with Qs_Tools;
use Qs_Tools;


with Integer_Util;


with Ada.Exceptions;
use Ada.Exceptions;

with Ada.Numerics.Aux;
use Ada.Numerics.Aux;
with Ada.Numerics.Float_Random;
use Ada.Numerics.Float_Random;

with Config;
use Config;
with Translate;
use Translate;

with Time_Units;
use Time_Units;

with Messages;
use Messages;
with Tasks;
use Tasks;
with Buffers;
use Buffers;
with Resources;
use Resources;


with Text_Io;
use Text_Io;

with Ada.Strings.Unbounded.Text_Io;
use Ada.Strings.Unbounded.Text_Io;


with Ada.Exceptions;
use Ada.Exceptions;


with Config;
use Config;
with Translate;
use Translate;

with Time_Units;
use Time_Units;

with Parameters;
use Parameters;
use Parameters.User_Defined_Parameters_Table_Package;


with Text_Io;
use Text_Io;


with Integer_Util;



package body Expressions is







   --------------------------------------------------


   procedure Put (
         S : in     Constant_Expr ) is 
   begin
      Put(To_Unbounded_String(S));
   end Put;

   procedure Put (
         S : in     Constant_Expr_Ptr ) is 
   begin
      Put(To_Unbounded_String(S.All));
   end Put;



   procedure Put (
         S : in     Variable_Expr ) is 
   begin
      Put(S.Name);
      Put("/");
      Put(S.Variable_Type'Img);
   end Put;

   procedure Put (
         S : in     Variable_Expr_Ptr ) is 
   begin
      Put(S.All);
   end Put;



   procedure Put (
         S : in     Array_Variable_Expr ) is 
   begin
      Put(S.Name);
      Put("/");
      Put(S.Variable_Type'Img);
      if (S.Array_Index/=null)  then
         Put(S.Array_Index.All);
      end if;
   end Put;

   procedure Put (
         S : in     Array_Variable_Expr_Ptr ) is 
   begin
      Put(S.All);
   end Put;


   procedure Put (
         S : in     Binary_Expr ) is 
   begin
      if (S.Lvalue/=null)  then
         Put(S.Lvalue.All);
      end if;
      Put(S.Ope);
      if (S.Rvalue/=null)  then
         Put(S.Rvalue.All);
      end if;
   end Put;

   procedure Put (
         S : in     Binary_Expr_Ptr ) is 
   begin
      Put(S.All);
   end Put;




   procedure Put (
         S : in     Unary_Expr ) is 
   begin
      Put(To_Unbounded_String(S));
   end Put;

   procedure Put (
         S : in     Unary_Expr_Ptr ) is 
   begin
      Put(To_Unbounded_String(S.All));
   end Put;


   procedure Put (
         S : in     Operator ) is 
   begin
      case S is
         when Plus=>
            Put(" + ");
         when Minus =>
            Put(" - ");
         when Div  =>
            Put(" / ");
         when Mult =>
            Put(" * ");
         when Exp =>
            Put(" ** ");
         when Modulo =>
            Put(" mod ");
         when Equal =>
            Put(" = ");
         when Not_Equal =>
            Put(" /= ");
         when Equal_Less =>
            Put(" <= ");
         when Equal_Greater =>
            Put(" >= ");
         when Inf =>
            Put(" < ");
         when Sup =>
            Put(" > ");
         when Logic_And =>
            Put(" and ");
         when Logic_Or =>
            Put(" or ");
         when Logic_Not =>
            Put(" not ");
         when Lcm =>
            Put(" lcm ");
         when Min_Operator =>
            Put(" min ");
         when Max_Operator =>
            Put(" max ");
         when Min_To_Index =>
            Put(" min_to_index ");
         when Max_To_Index =>
            Put(" max_to_index ");
         when Abs_Operator =>
            Put(" abs ");
         when Get_Buffer_Index =>
            Put(" get_buffer_index ");
         when Get_Message_Index =>
            Put(" get_message_index ");
         when Get_Resource_Index =>
            Put(" get_resource_index ");
         when Get_Task_Index =>
            Put(" get_task_index ");
         when Concat =>
            Put(" & ");
      end case;
   end Put;




   ------------------------------------------------------


   procedure Initialize (
         S : in out Constant_Expr ) is 
   begin
      S.Expression_Type:=Constant_Type;
   end Initialize;


   procedure Initialize (
         S : in out Variable_Expr ) is 
   begin
      S.Expression_Type:=Variable_Type;
   end Initialize;


   procedure Initialize (
         S : in out Array_Variable_Expr ) is 
   begin
      S.Expression_Type:=Array_Variable_Type;
   end Initialize;


   procedure Initialize (
         S : in out Binary_Expr ) is 
   begin
      S.Expression_Type:=Binary_Type;
   end Initialize;


   procedure Initialize (
         S : in out Unary_Expr ) is 
   begin
      S.Expression_Type:=Unary_Type;
   end Initialize;



   ---------------------------------------------------------------



   function To_Unbounded_String (
         C : Constant_Expr ) 
     return Unbounded_String is 
   begin
      return     To_Unbounded_String(C.Value);
   end To_Unbounded_String;


   function To_Unbounded_String (
         C : Variable_Expr ) 
     return Unbounded_String is 
   begin
      return C.Name;
   end To_Unbounded_String;


   function To_Unbounded_String (
         C : Array_Variable_Expr ) 
     return Unbounded_String is 
   begin
      return C.Name;
   end To_Unbounded_String;


   function To_Unbounded_String (
         C : Binary_Expr ) 
     return Unbounded_String is 
   begin
      return (  To_Unbounded_String(C.Lvalue.All) &
         To_Unbounded_String(C.
            Ope'Img) &
         To_Unbounded_String(C.Rvalue.All) );
   end To_Unbounded_String;


   function To_Unbounded_String (
         C : Unary_Expr ) 
     return Unbounded_String is 
   begin
      return (  To_Unbounded_String(C.Ope'Img)
         &  To_Unbounded_String(C.Value.All) );
   end To_Unbounded_String;





   ---------------------------------------------------------------



   function Get_Type (
         C : Constant_Expr ) 
     return Simulation_Type is 
   begin
      return C.Value.Ptype;
   end Get_Type;


   function Get_Type (
         C : Variable_Expr ) 
     return Simulation_Type is 
   begin
      return C.Variable_Type;
   end Get_Type;



   function Get_Type (
         C : Binary_Expr ) 
     return Simulation_Type is 
   begin
      return Get_Type(C.Rvalue.All);
   end Get_Type;


   function Get_Type (
         C : Unary_Expr ) 
     return Simulation_Type is 
   begin
      return Get_Type(C.Value.All);
   end Get_Type;



   procedure Put (
         V : in     Variable_Record_Ptr ) is 
   begin
      Put(V.Variable);
      Put(V.Simulation);
   end Put;


   function Find_Variable (
         Var_Table : in     Variables_Table_Type; 
         S         : in     Unbounded_String      ) 
     return Variables_Range is 

   begin
      for I in 0..Var_Table.Nb_Entries-1 loop
         if Var_Table.Entries(I).Variable.Name=S
               then
            return I;
         end if;
      end loop;

      Raise_Exception(Expressions.Syntax_Error'Identity, To_String(
            S & Lb_Comma & Lb_Undeclared_Identifier(Current_Language) ));

   end Find_Variable;


   function Find_Variable (
         Var_Table : in     Variables_Table_Type; 
         S         : in     String                ) 
     return Variables_Range is 
   begin
      return Find_Variable(Var_Table, To_Unbounded_String(S));
   end Find_Variable;


   procedure Check_Variable_Declaration (
         Var_Table : in     Variables_Table_Type; 
         S         : in     Unbounded_String      ) is 

      Dummy : Variables_Range;  

   begin
      Dummy:=Find_Variable(Var_Table,S);
   end Check_Variable_Declaration;



   procedure Load_Parametric_Variables (
         Var_Table   : in out Variables_Table_Type; 
         My_Messages : in     Messages_Set          ) is 

      Var       : Variables_Range;  
      A_Message : Message_Ptr;  
      Iterator  : Messages_Iterator;  
      Index     : Simulations_Range;  

   begin

      ---------------------------------------------------------------
      --   Set message variables
      ---------------------------------------------------------------


      -- "nb_messages"
      --
      Var:=Find_Variable(Var_Table,
         To_Unbounded_String("nb_messages"));
      Var_Table.Entries(Var).Simulation.
         Integer_Value:=Integer(Get_Number_Of_Elements(My_Messages));


      if not Is_Empty(My_Messages)
            then
         Reset_Iterator(My_Messages, Iterator);
         Index:=0;

         loop
            Current_Element(My_Messages, A_Message, Iterator);


            -- "name"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("messages.name"));
            Var_Table.Entries(Var).Simulation.
               String_Table_Value(Index):=
               A_Message.Name;


            -- "deadline"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("messages.deadline"));
            Var_Table.Entries(Var).Simulation.
               Integer_Table_Value(Index):=
               Integer(A_Message.Deadline);


            -- "delay"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("messages.delay"));
            Var_Table.Entries(Var).Simulation.
               Integer_Table_Value(Index):=
               Integer(A_Message.Response_Time);


            -- "size"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("messages.size"));
            Var_Table.Entries(Var).Simulation.
               Integer_Table_Value(Index):=
               Integer(A_Message.Size);


            -- "period" and "jitter"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("messages.period"));
            case A_Message.Message_Type is
               when Aperiodic_Type =>
                  Var_Table.Entries(Var).Simulation.
                     Integer_Table_Value(Index):=0;
               when others =>
                  Var_Table.Entries(Var).Simulation.
                     Integer_Table_Value(Index):=
                     Integer(Periodic_Message_Ptr(A_Message).Period);
            end case;
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("messages.jitter"));
            case A_Message.Message_Type is
               when Aperiodic_Type =>
                  Var_Table.Entries(Var).Simulation.
                     Integer_Table_Value(Index):=0;
               when others =>
                  Var_Table.Entries(Var).Simulation.
                     Integer_Table_Value(Index):=
                     Integer(Periodic_Message_Ptr(A_Message).Jitter);
            end case;


            -- "ready"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("messages.ready"));
            Var_Table.Entries(Var).Simulation.
               Boolean_Table_Value(Index):=False;


            exit when Is_Last_Element(My_Messages, Iterator);
            Next_Element(My_Messages, Iterator);
            Index:=Index+1;

         end loop;
      end if;

   end Load_Parametric_Variables;





   procedure Load_Parametric_Variables (
         Var_Table  : in out Variables_Table_Type; 
         My_Buffers : in     Buffers_Set           ) is 


      Var      : Variables_Range;  
      A_Buffer : Buffer_Ptr;  
      Iterator : Buffers_Iterator;  
      Index    : Simulations_Range;  

   begin

      ---------------------------------------------------------------
      --   Set buffer variables
      ---------------------------------------------------------------


      -- "nb_buffers"
      --
      Var:=Find_Variable(Var_Table,
         To_Unbounded_String("nb_buffers"));
      Var_Table.Entries(Var).Simulation.
         Integer_Value:=Integer(Get_Number_Of_Elements(My_Buffers));


      if not Is_Empty(My_Buffers)
            then
         Reset_Iterator(My_Buffers, Iterator);
         Index:=0;

         loop
            Current_Element(My_Buffers, A_Buffer, Iterator);


            -- "processor_name"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("buffers.processor_name"));
            Var_Table.Entries(Var).Simulation.
               String_Table_Value(Index):=
               A_Buffer.Cpu_Name;



            -- "name"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("buffers.name"));
            Var_Table.Entries(Var).Simulation.
               String_Table_Value(Index):=
               A_Buffer.Name;



            -- "max_size"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("buffers.max_size"));
            Var_Table.Entries(Var).Simulation.
               Integer_Table_Value(Index):=
               Integer(A_Buffer.Size);



            exit when Is_Last_Element(My_Buffers, Iterator);
            Next_Element(My_Buffers, Iterator);
            Index:=Index+1;

         end loop;
      end if;

   end Load_Parametric_Variables;




   procedure Load_Parametric_Variables (
         Var_Table    : in out Variables_Table_Type; 
         My_Resources : in     Resources_Set         ) is 


      Var        : Variables_Range;  
      A_Resource : Resource_Ptr;  
      Iterator   : Resources_Iterator;  
      Index      : Simulations_Range;  

   begin


      ---------------------------------------------------------------
      --   Set resource variables
      ---------------------------------------------------------------


      -- "nb_resources"
      --
      Var:=Find_Variable(Var_Table,
         To_Unbounded_String("nb_resources"));
      Var_Table.Entries(Var).Simulation.
         Integer_Value:=Integer(Get_Number_Of_Elements(My_Resources));


      if not Is_Empty(My_Resources)
            then
         Reset_Iterator(My_Resources, Iterator);
         Index:=0;

         loop
            Current_Element(My_Resources, A_Resource, Iterator);


            -- "name"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("resources.name"));
            Var_Table.Entries(Var).Simulation.
               String_Table_Value(Index):=
               A_Resource.Name;


            -- "initial_state"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("resources.initial_state"));
            Var_Table.Entries(Var).Simulation.
               Integer_Table_Value(Index):=
               A_Resource.State;


            -- "protocol"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("resources.protocol"));
            Var_Table.Entries(Var).Simulation.
               String_Table_Value(Index):=
               To_Unbounded_String(A_Resource.Protocol'Img);


            -- "processor_name"
            --
            Var:=Find_Variable(Var_Table,
               To_Unbounded_String("resources.processor_name"));
            Var_Table.Entries(Var).Simulation.
               String_Table_Value(Index):=
               A_Resource.Cpu_Name;



            exit when Is_Last_Element(My_Resources, Iterator);
            Next_Element(My_Resources, Iterator);
            Index:=Index+1;

         end loop;
      end if;

   end Load_Parametric_Variables;



   procedure Load_Parametric_Variables (
         Var_Table : in out Variables_Table_Type; 
         My_Tasks  : in     Tasks_Set             ) is 


      Var      : Variables_Range;  
      A_Task   : Task_Ptr;  
      Iterator : Tasks_Iterator;  
      Index    : Simulations_Range;  

   begin

      ---------------------------------------------------------------
      --   Set task variables
      ---------------------------------------------------------------


      -- "nb_tasks"
      --
      Var:=Find_Variable(Var_Table,
         To_Unbounded_String("nb_tasks"));
      Var_Table.Entries(Var).Simulation.
         Integer_Value:=Integer(Get_Number_Of_Elements(My_Tasks));

      Reset_Iterator(My_Tasks, Iterator);
      Index:=0;

      loop
         Current_Element(My_Tasks, A_Task, Iterator);


         -- "priority"
         --
         Var:=Find_Variable(Var_Table,
            To_Unbounded_String("tasks.priority"));
         Var_Table.Entries(Var).Simulation.
            Integer_Table_Value(Index):=
            Integer(A_Task.Priority);


         -- "deadline"
         --
         Var:=Find_Variable(Var_Table,
            To_Unbounded_String("tasks.deadline"));
         Var_Table.Entries(Var).Simulation.
            Integer_Table_Value(Index):=
            Integer(A_Task.Deadline);


         -- "start_time"
         --
         Var:=Find_Variable(Var_Table,
            To_Unbounded_String("tasks.start_time"));
         Var_Table.Entries(Var).Simulation.
            Integer_Table_Value(Index):=
            Integer(A_Task.Start_Time);


         -- "processor_name"
         --
         Var:=Find_Variable(Var_Table,
            To_Unbounded_String("tasks.processor_name"));
         Var_Table.Entries(Var).Simulation.
            String_Table_Value(Index):=
            A_Task.Cpu_Name;


         -- "name"
         --
         Var:=Find_Variable(Var_Table,
            To_Unbounded_String("tasks.name"));
         Var_Table.Entries(Var).Simulation.
            String_Table_Value(Index):=
            A_Task.Name;


         -- "capacity"
         --
         Var:=Find_Variable(Var_Table,
            To_Unbounded_String("tasks.capacity"));
         Var_Table.Entries(Var).Simulation.
            Integer_Table_Value(Index):=
            Integer(A_Task.Capacity);


         -- "blocking_time"
         --
         Var:=Find_Variable(Var_Table,
            To_Unbounded_String("tasks.blocking_time"));
         Var_Table.Entries(Var).Simulation.
            Integer_Table_Value(Index):=
            Integer(A_Task.Blocking_Time);


         -- "period"
         --
         Var:=Find_Variable(Var_Table,
            To_Unbounded_String("tasks.period"));
         case A_Task.Task_Type is
            when Aperiodic_Type =>
               Var_Table.Entries(Var).Simulation.
                  Integer_Table_Value(Index):=0;
            when others =>
               Var_Table.Entries(Var).Simulation.
                  Integer_Table_Value(Index):=
                  Integer(Periodic_Task_Ptr(A_Task).Period);
         end case;


         -- "jitter"
         --
         Var:=Find_Variable(Var_Table,
            To_Unbounded_String("tasks.jitter"));
         case A_Task.Task_Type is
            when Aperiodic_Type =>
               Var_Table.Entries(Var).Simulation.
                  Integer_Table_Value(Index):=0;
            when others =>
               Var_Table.Entries(Var).Simulation.
                  Integer_Table_Value(Index):=
                  Integer(Periodic_Task_Ptr(A_Task).Jitter);
         end case;



         -- user defined task's parameter :
         --

         for I in 0..A_Task.Param.Nb_Entries-1 loop

            -- Find the user defined variable
            -- and set its value
            --
            for J in 0..Var_Table.Nb_Entries-1 loop
               if Var_Table.Entries(J).Variable.Name=
                     To_Unbounded_String("tasks.") & A_Task.Param.Entries(
                     I).Name
                     then

                  -- Now, set the value
                  --
                  if A_Task.Param.Entries(I).Ptype=
                        Boolean_Parameter then
                     Var_Table.Entries(J).
                        Simulation.
                        Boolean_Table_Value(Index):=
                        Boolean(A_Task.
                        Param.Entries(I).Boolean_Value);
                  end if;
                  if A_Task.Param.Entries(I).Ptype=
                        Integer_Parameter then
                     Var_Table.Entries(J).
                        Simulation.
                        Integer_Table_Value(Index):=
                        Integer(A_Task.
                        Param.Entries(I).Integer_Value);
                  end if;
                  if A_Task.Param.Entries(I).Ptype=
                        Double_Parameter then
                     Var_Table.Entries(J).
                        Simulation.
                        Double_Table_Value(Index):=Double(
                        A_Task.
                        Param.Entries(I).Double_Value);
                  end if;
                  if A_Task.Param.Entries(I).Ptype=
                        String_Parameter then
                     Var_Table.Entries(J).
                        Simulation.
                        String_Table_Value(Index):=
                        A_Task.
                        Param.Entries(I).String_Value;
                  end if;
               end if;
            end loop;
         end loop;


         exit when Is_Last_Element(My_Tasks, Iterator);
         Next_Element(My_Tasks, Iterator);
         Index:=Index+1;

      end loop;



   end Load_Parametric_Variables;












   function Apply_Random_Operator (
         Seed       : in     Generator;                 
         Parameters : in     Random_Law_Parameters_Type ) 
     return Integer; 

   function Apply_Operator (
         Lvalue : Integer; 
         Rvalue : Integer; 
         Ope    : Operator ) 
     return Integer; 

   function Apply_Operator (
         Lvalue : Double;  
         Rvalue : Double;  
         Ope    : Operator ) 
     return Double; 

   function Apply_Operator (
         Lvalue : Double;  
         Rvalue : Double;  
         Ope    : Operator ) 
     return Boolean; 

   function Apply_Operator (
         Lvalue : Unbounded_String; 
         Rvalue : Unbounded_String; 
         Ope    : Operator          ) 
     return Unbounded_String; 

   function Apply_Operator (
         Lvalue : Unbounded_String; 
         Rvalue : Unbounded_String; 
         Ope    : Operator          ) 
     return Boolean; 

   function Apply_Operator (
         Lvalue : Integer; 
         Rvalue : Integer; 
         Ope    : Operator ) 
     return Boolean; 

   function Apply_Operator (
         Lvalue : Boolean; 
         Rvalue : Boolean; 
         Ope    : Operator ) 
     return Boolean; 

   function Compute_Binary_Value (
         Lvalue : Simulation_Value_Ptr; 
         Rvalue : Simulation_Value_Ptr; 
         Ope    : Operator              ) 
     return Simulation_Value_Ptr; 

   function Apply_Operator (
         Value : Boolean; 
         Ope   : Operator ) 
     return Boolean; 

   function Apply_Operator (
         Value : Integer; 
         Ope   : Operator ) 
     return Integer; 

   function Apply_Operator (
         Value : Double;  
         Ope   : Operator ) 
     return Double; 

   function Apply_Operator (
         Value : Unbounded_String;     
         V     : Variables_Table_Type; 
         Ope   : Operator              ) 
     return Integer; 




   function Apply_Random_Operator (
         Seed       : in     Generator;                 
         Parameters : in     Random_Law_Parameters_Type ) 
     return Integer is 

      Avg   : Integer;  
      Value : Integer;  

   begin

      case Parameters.Law is

         when Exponential_Law =>
            Value:=Integer(Get_Exponential_Time(Double(Parameters.
                     Parameter1), Seed));

         when Uniform_Law =>
            Avg:=(Parameters.Parameter1+Parameters.Parameter2)/2;
            Value:=Integer((Double(Random(Seed))+0.5)*Double(Avg));
      end case;

      return Value;

   end Apply_Random_Operator;




   function Apply_Operator (
         Lvalue : Integer; 
         Rvalue : Integer; 
         Ope    : Operator ) 
     return Integer is 

   begin

      if Ope=Min_Operator then
         if Lvalue<Rvalue then
            return Lvalue;
         else
            return Rvalue;
         end if;
      end if;
      if Ope=Max_Operator then
         if Lvalue>Rvalue then
            return Lvalue;
         else
            return Rvalue;
         end if;
      end if;
      if Ope=Lcm then
         return Integer_Util.Lcm(Lvalue,Rvalue);
      end if;
      if Ope=Plus then
         return Lvalue+Rvalue;
      end if;
      if Ope=Minus then
         return Lvalue-Rvalue;
      end if;
      if Ope=Mult then
         return Lvalue*Rvalue;
      end if;
      if Ope=Div then
         return Lvalue/Rvalue;
      end if;
      if Ope=Modulo then
         return Lvalue mod Rvalue;
      end if;


      Raise_Exception(Type_Error'Identity,
         To_String(Lb_Uncompatible_Type_Error(Current_Language) &
            Lb_Colon & To_Unbounded_String(Lvalue'Img) &
            Lb_Comma  & To_Unbounded_String(Rvalue'Img) &
            Lb_Comma &
            To_Unbounded_String(Ope'Img)  ) );


      -- Arithmetic exception : return 0 in any cases
      -- to avoid stopping the application
      --
   exception
      when others =>
         return 0;

   end Apply_Operator;


   function Apply_Operator (
         Lvalue : Double;  
         Rvalue : Double;  
         Ope    : Operator ) 
     return Double is 
   begin

      if Ope=Min_Operator then
         if Lvalue<Rvalue then
            return Lvalue;
         else
            return Rvalue;
         end if;
      end if;
      if Ope=Max_Operator then
         if Lvalue>Rvalue then
            return Lvalue;
         else
            return Rvalue;
         end if;
      end if;
      if Ope=Lcm then
         return Double(Integer_Util.Lcm(Integer(Lvalue),Integer(Rvalue)));
      end if;
      if Ope=Plus then
         return Lvalue+Rvalue;
      end if;
      if Ope=Minus then
         return Lvalue-Rvalue;
      end if;
      if Ope=Mult then
         return Lvalue*Rvalue;
      end if;

      if Ope=Div then
         return Lvalue/Rvalue;
      end if;

      Raise_Exception(Type_Error'Identity,
         To_String(Lb_Uncompatible_Type_Error(Current_Language) &
            Lb_Colon & To_Unbounded_String(Lvalue'Img) &
            Lb_Comma  & To_Unbounded_String(Rvalue'Img) &
            Lb_Comma &
            To_Unbounded_String(Ope'Img)  ) );


      -- Arithmetic exception : return 0 in any cases
      -- to avoid stopping the application
      --
   exception
      when others =>
         return 0.0;
   end Apply_Operator;


   function Apply_Operator (
         Lvalue : Double;  
         Rvalue : Double;  
         Ope    : Operator ) 
     return Boolean is 
   begin
      if Ope=Equal then
         return Lvalue=Rvalue;
      end if;
      if Ope=Not_Equal then
         return Lvalue/=Rvalue;
      end if;
      if Ope=Equal_Less then
         return Lvalue<=Rvalue;
      end if;
      if Ope=Equal_Greater then
         return Lvalue>=Rvalue;
      end if;
      if Ope=Inf then
         return Lvalue<Rvalue;
      end if;
      if Ope=Sup then
         return Lvalue>Rvalue;
      end if;

      Raise_Exception(Type_Error'Identity,
         To_String(Lb_Uncompatible_Type_Error(Current_Language) &
            Lb_Colon & To_Unbounded_String(Lvalue'Img) &
            Lb_Comma  & To_Unbounded_String(Rvalue'Img) &
            Lb_Comma &
            To_Unbounded_String(Ope'Img)  ) );


   end Apply_Operator;


   function Apply_Operator (
         Lvalue : Unbounded_String; 
         Rvalue : Unbounded_String; 
         Ope    : Operator          ) 
     return Unbounded_String is 
   begin
      if Ope=Concat
            then
         return Lvalue & Rvalue;
      end if;


      Raise_Exception(Type_Error'Identity,
         To_String(Lb_Uncompatible_Type_Error(Current_Language) &
            Lb_Colon & Lvalue &
            Lb_Comma  & Rvalue &
            Lb_Comma &
            To_Unbounded_String(Ope'Img)) );

   end Apply_Operator;



   function Apply_Operator (
         Lvalue : Unbounded_String; 
         Rvalue : Unbounded_String; 
         Ope    : Operator          ) 
     return Boolean is 
   begin
      if Ope=Equal then
         return Lvalue=Rvalue;
      end if;
      if Ope=Not_Equal then
         return Lvalue/=Rvalue;
      end if;
      if Ope=Equal_Less then
         return Lvalue<=Rvalue;
      end if;
      if Ope=Equal_Greater then
         return Lvalue>=Rvalue;
      end if;
      if Ope=Inf then
         return Lvalue<Rvalue;
      end if;
      if Ope=Sup then
         return Lvalue>Rvalue;
      end if;

      Raise_Exception(Type_Error'Identity,
         To_String(Lb_Uncompatible_Type_Error(Current_Language) &
            Lb_Colon & Lvalue &
            Lb_Comma  & Rvalue &
            Lb_Comma &
            To_Unbounded_String(Ope'Img)  ) );

   end Apply_Operator;


   function Apply_Operator (
         Lvalue : Integer; 
         Rvalue : Integer; 
         Ope    : Operator ) 
     return Boolean is 
   begin
      if Ope=Equal then
         return Lvalue=Rvalue;
      end if;
      if Ope=Not_Equal then
         return Lvalue/=Rvalue;
      end if;
      if Ope=Equal_Less then
         return Lvalue<=Rvalue;
      end if;
      if Ope=Equal_Greater then
         return Lvalue>=Rvalue;
      end if;
      if Ope=Inf then
         return Lvalue<Rvalue;
      end if;

      if Ope=Sup then
         return Lvalue>Rvalue;
      end if;


      Raise_Exception(Type_Error'Identity,
         To_String(Lb_Uncompatible_Type_Error(Current_Language) &
            Lb_Colon & To_Unbounded_String(Lvalue'Img) &
            Lb_Comma  & To_Unbounded_String(Rvalue'Img) &
            Lb_Comma &
            To_Unbounded_String(Ope'Img)  ) );

   end Apply_Operator;


   function Apply_Operator (
         Lvalue : Boolean; 
         Rvalue : Boolean; 
         Ope    : Operator ) 
     return Boolean is 
   begin

      if Ope=Equal then
         return Lvalue=Rvalue;
      end if;
      if Ope=Not_Equal then
         return Lvalue/=Rvalue;
      end if;
      if Ope=Equal_Less then
         return Lvalue<=Rvalue;
      end if;
      if Ope=Equal_Greater then
         return Lvalue>=Rvalue;
      end if;
      if Ope=Inf then
         return Lvalue<Rvalue;
      end if;
      if Ope=Sup then
         return Lvalue>Rvalue;
      end if;
      if Ope=Logic_And then
         return Lvalue and Rvalue;
      end if;
      if Ope=Logic_Or then
         return Lvalue or Rvalue;
      end if;


      Raise_Exception(Type_Error'Identity,
         To_String(Lb_Uncompatible_Type_Error(Current_Language) &
            Lb_Colon & To_Unbounded_String(Lvalue'Img) &
            Lb_Comma  & To_Unbounded_String(Rvalue'Img) &
            Lb_Comma &
            To_Unbounded_String(Ope'Img)  ) );


   end Apply_Operator;





   function Compute_Binary_Value (
         Lvalue : Simulation_Value_Ptr; 
         Rvalue : Simulation_Value_Ptr; 
         Ope    : Operator              ) 
     return Simulation_Value_Ptr is 

      Res : Simulation_Value_Ptr;  

   begin

      case Lvalue.Ptype  is



         -- Nothing to compute in the cases of random type
         -- because value_of functions already do the job
         -- by returning integer values
         --
         when Simulation_Random =>
            null;

         when Simulation_Array_Random =>
            null;

         when Simulation_Time_Unit_Array_Random =>
            null;




         when Simulation_Integer =>

            if Rvalue.Ptype=Simulation_Integer then

               if (Ope=Equal) or
                     (Ope=Not_Equal) or
                     (Ope=Equal_Less) or
                     (Ope=Equal_Greater) or
                     (Ope=Inf) or
                     (Ope=Sup) then
                  Res:=new Simulation_Value(Simulation_Boolean);
                  Res.Boolean_Value:=Apply_Operator(Lvalue.Integer_Value,
                     Rvalue.Integer_Value, Ope);
               else
                  Res:=new Simulation_Value(Simulation_Integer);
                  Res.Integer_Value:=Apply_Operator(Lvalue.Integer_Value,
                     Rvalue.Integer_Value, Ope);
               end if;

            else
               if Rvalue.Ptype=Simulation_Array_Integer then
                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res:=new Simulation_Value(
                        Simulation_Array_Boolean);
                  else
                     Res:=new Simulation_Value(
                        Simulation_Array_Integer);
                  end if;

                  for I in Simulations_Range loop
                     if (Ope=Equal) or
                           (Ope=Not_Equal) or
                           (Ope=Equal_Less) or
                           (Ope=Equal_Greater) or
                           (Ope=Inf) or
                           (Ope=Sup) then
                        Res.Boolean_Table_Value(I):=Apply_Operator(
                           Lvalue.Boolean_Value,
                           Rvalue.Boolean_Table_Value(I),Ope);
                     else
                        Res.Integer_Table_Value(I):=Apply_Operator(
                           Lvalue.Integer_Value,
                           Rvalue.Integer_Table_Value(I),Ope);
                     end if;
                  end loop;

               else

                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res:=new Simulation_Value(
                        Simulation_Time_Unit_Array_Boolean);
                  else
                     Res:=new Simulation_Value(
                        Simulation_Time_Unit_Array_Integer);
                  end if;

                  for I in Time_Unit_Range loop
                     if (Ope=Equal) or
                           (Ope=Not_Equal) or
                           (Ope=Equal_Less) or
                           (Ope=Equal_Greater) or
                           (Ope=Inf) or
                           (Ope=Sup) then
                        Res.Boolean_Time_Unit_Table_Value(I):=
                           Apply_Operator(
                           Lvalue.Boolean_Value,
                           Rvalue.Boolean_Time_Unit_Table_Value(I),Ope);
                     else
                        Res.Integer_Time_Unit_Table_Value(I):=
                           Apply_Operator(
                           Lvalue.Integer_Value,
                           Rvalue.Integer_Time_Unit_Table_Value(I),Ope);
                     end if;
                  end loop;
               end if;
            end if;


         when Simulation_String =>

            if Rvalue.Ptype=Simulation_String then

               if (Ope=Equal) or
                     (Ope=Not_Equal) or
                     (Ope=Equal_Less) or
                     (Ope=Equal_Greater) or
                     (Ope=Inf) or
                     (Ope=Sup) then
                  Res:=new Simulation_Value(Simulation_Boolean);
                  Res.Boolean_Value:=Apply_Operator(Lvalue.String_Value,
                     Rvalue.String_Value, Ope);
               else
                  Res:=new Simulation_Value(Simulation_String);
                  Res.String_Value:=Apply_Operator(Lvalue.String_Value,
                     Rvalue.String_Value, Ope);
               end if;

            else
               if Rvalue.Ptype=Simulation_Array_String then
                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res:=new Simulation_Value(
                        Simulation_Array_Boolean);
                  else
                     Res:=new Simulation_Value(
                        Simulation_Array_String);
                  end if;

                  for I in Simulations_Range loop
                     if (Ope=Equal) or
                           (Ope=Not_Equal) or
                           (Ope=Equal_Less) or
                           (Ope=Equal_Greater) or
                           (Ope=Inf) or
                           (Ope=Sup) then
                        Res.Boolean_Table_Value(I):=Apply_Operator(
                           Lvalue.Boolean_Value,
                           Rvalue.Boolean_Table_Value(I),Ope);
                     else
                        Res.String_Table_Value(I):=Apply_Operator(
                           Lvalue.String_Value,
                           Rvalue.String_Table_Value(I),Ope);
                     end if;
                  end loop;

               else

                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res:=new Simulation_Value(
                        Simulation_Time_Unit_Array_Boolean);
                  else
                     Res:=new Simulation_Value(
                        Simulation_Time_Unit_Array_String);
                  end if;

                  for I in Time_Unit_Range loop
                     if (Ope=Equal) or
                           (Ope=Not_Equal) or
                           (Ope=Equal_Less) or
                           (Ope=Equal_Greater) or
                           (Ope=Inf) or
                           (Ope=Sup) then
                        Res.Boolean_Time_Unit_Table_Value(I):=
                           Apply_Operator(
                           Lvalue.Boolean_Value,
                           Rvalue.Boolean_Time_Unit_Table_Value(I),Ope);
                     else
                        Res.String_Time_Unit_Table_Value(I):=
                           Apply_Operator(
                           Lvalue.String_Value,
                           Rvalue.String_Time_Unit_Table_Value(I),Ope);
                     end if;
                  end loop;
               end if;
            end if;





         when Simulation_Double =>


            if Rvalue.Ptype=Simulation_Double then

               if (Ope=Equal) or
                     (Ope=Not_Equal) or
                     (Ope=Equal_Less) or
                     (Ope=Equal_Greater) or
                     (Ope=Inf) or
                     (Ope=Sup) then
                  Res:=new Simulation_Value(Simulation_Boolean);
                  Res.Boolean_Value:=Apply_Operator(Lvalue.Double_Value,
                     Rvalue.Double_Value, Ope);
               else
                  Res:=new Simulation_Value(Simulation_Double);
                  Res.Double_Value:=Apply_Operator(Lvalue.Double_Value,
                     Rvalue. Double_Value, Ope);
               end if;

            else
               if Rvalue.Ptype=Simulation_Array_Double then
                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res:=new Simulation_Value(
                        Simulation_Array_Boolean);
                  else
                     Res:=new Simulation_Value(
                        Simulation_Array_Double);
                  end if;

                  for I in Simulations_Range loop
                     if (Ope=Equal) or
                           (Ope=Not_Equal) or
                           (Ope=Equal_Less) or
                           (Ope=Equal_Greater) or
                           (Ope=Inf) or
                           (Ope=Sup) then
                        Res.Boolean_Table_Value(I):=Apply_Operator(
                           Lvalue.Boolean_Value,
                           Rvalue.Boolean_Table_Value(I),Ope);
                     else
                        Res.Double_Table_Value(I):=Apply_Operator(
                           Lvalue.Double_Value,
                           Rvalue.Double_Table_Value(I),Ope);
                     end if;
                  end loop;

               else

                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res:=new Simulation_Value(
                        Simulation_Time_Unit_Array_Boolean);
                  else
                     Res:=new Simulation_Value(
                        Simulation_Time_Unit_Array_Double);
                  end if;

                  for I in Time_Unit_Range loop
                     if (Ope=Equal) or
                           (Ope=Not_Equal) or
                           (Ope=Equal_Less) or
                           (Ope=Equal_Greater) or
                           (Ope=Inf) or
                           (Ope=Sup) then
                        Res.Boolean_Time_Unit_Table_Value(I):=
                           Apply_Operator(
                           Lvalue.Boolean_Value,
                           Rvalue.Boolean_Time_Unit_Table_Value(I),Ope);
                     else
                        Res.Double_Time_Unit_Table_Value(I):=
                           Apply_Operator(
                           Lvalue.Double_Value,
                           Rvalue.Double_Time_Unit_Table_Value(I),Ope);
                     end if;
                  end loop;
               end if;
            end if;






         when Simulation_Boolean =>


            if Rvalue.Ptype=Simulation_Boolean then

               Res:=new Simulation_Value(Simulation_Boolean);
               Res.Boolean_Value:=Apply_Operator(Lvalue.Boolean_Value,
                  Rvalue.Boolean_Value, Ope);
            else
               if Rvalue.Ptype=Simulation_Array_Boolean then

                  Res:=new Simulation_Value(
                     Simulation_Array_Boolean);

                  for I in Simulations_Range loop
                     Res.Boolean_Table_Value(I):=Apply_Operator(
                        Lvalue.Boolean_Value,
                        Rvalue.Boolean_Table_Value(I),Ope);
                  end loop;

               else

                  Res:=new Simulation_Value(
                     Simulation_Time_Unit_Array_Boolean);

                  for I in Time_Unit_Range loop
                     Res.Boolean_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.Boolean_Value,
                        Rvalue.Boolean_Time_Unit_Table_Value(I),Ope);
                  end loop;

               end if;
            end if;





         when Simulation_Array_Integer =>


            if (Ope=Equal) or
                  (Ope=Not_Equal) or
                  (Ope=Equal_Less) or
                  (Ope=Equal_Greater) or
                  (Ope=Inf) or
                  (Ope=Sup) then
               Res:=new Simulation_Value(
                  Simulation_Array_Boolean);
            else
               Res:=new Simulation_Value(
                  Simulation_Array_Integer);
            end if;


            for I in Simulations_Range loop

               if Rvalue.Ptype=Simulation_Array_Integer then

                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Table_Value(I):=Apply_Operator(Lvalue.
                        Boolean_Table_Value(I), Rvalue.
                        Boolean_Table_Value(I), Ope);
                  else
                     Res.Integer_Table_Value(I):=Apply_Operator(Lvalue.
                        Integer_Table_Value(I), Rvalue.
                        Integer_Table_Value(I), Ope);
                  end if;

               else

                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Table_Value(I):=Apply_Operator(Lvalue.
                        Boolean_Table_Value(I), Rvalue.Boolean_Value, Ope);
                  else
                     Res.Integer_Table_Value(I):=Apply_Operator(Lvalue.
                        Integer_Table_Value(I), Rvalue.Integer_Value, Ope);
                  end if;

               end if;

            end loop;



         when Simulation_Array_Double =>


            if (Ope=Equal) or
                  (Ope=Not_Equal) or
                  (Ope=Equal_Less) or
                  (Ope=Equal_Greater) or
                  (Ope=Inf) or
                  (Ope=Sup) then
               Res:=new Simulation_Value(
                  Simulation_Array_Boolean);
            else
               Res:=new Simulation_Value(
                  Simulation_Array_Double);
            end if;

            for I in Simulations_Range loop

               if Rvalue.Ptype=Simulation_Array_Double then
                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Table_Value(I):=Apply_Operator(Lvalue.
                        Boolean_Table_Value(I), Rvalue.
                        Boolean_Table_Value(I),Ope);
                  else
                     Res.Double_Table_Value(I):=Apply_Operator(Lvalue.
                        Double_Table_Value(I), Rvalue.Double_Table_Value(
                           I),Ope);
                  end if;
               else
                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Table_Value(I):=Apply_Operator(Lvalue.
                        Boolean_Table_Value(I), Rvalue.Boolean_Value, Ope);
                  else
                     Res.Double_Table_Value(I):=Apply_Operator(Lvalue.
                        Double_Table_Value(I), Rvalue.Double_Value, Ope);
                  end if;
               end if;

            end loop;




         when Simulation_Array_String =>


            if (Ope=Equal) or
                  (Ope=Not_Equal) or
                  (Ope=Equal_Less) or
                  (Ope=Equal_Greater) or
                  (Ope=Inf) or
                  (Ope=Sup) then
               Res:=new Simulation_Value(
                  Simulation_Array_Boolean);
            else
               Res:=new Simulation_Value(
                  Simulation_Array_String);
            end if;

            for I in Simulations_Range loop

               if Rvalue.Ptype=Simulation_Array_String then
                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Table_Value(I):=Apply_Operator(Lvalue.
                        Boolean_Table_Value(I), Rvalue.
                        Boolean_Table_Value(I),Ope);
                  else
                     Res.String_Table_Value(I):=Apply_Operator(Lvalue.
                        String_Table_Value(I), Rvalue.String_Table_Value(
                           I),Ope);
                  end if;
               else
                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Table_Value(I):=Apply_Operator(Lvalue.
                        Boolean_Table_Value(I), Rvalue.Boolean_Value, Ope);
                  else
                     Res.String_Table_Value(I):=Apply_Operator(Lvalue.
                        String_Table_Value(I), Rvalue.String_Value, Ope);
                  end if;
               end if;

            end loop;




         when Simulation_Array_Boolean =>


            Res:=new Simulation_Value(
               Simulation_Array_Boolean);

            for I in Simulations_Range loop

               if Rvalue.Ptype=Simulation_Array_Boolean then
                  Res.Boolean_Table_Value(I):=Apply_Operator(
                     Lvalue.Boolean_Table_Value(I),
                     Rvalue.Boolean_Table_Value(I),Ope);
               else
                  Res.Boolean_Table_Value(I):=Apply_Operator(
                     Lvalue.
                     Boolean_Table_Value(I),
                     Rvalue.Boolean_Value, Ope);
               end if;

            end loop;




         when Simulation_Time_Unit_Array_Integer =>


            if (Ope=Equal) or
                  (Ope=Not_Equal) or
                  (Ope=Equal_Less) or
                  (Ope=Equal_Greater) or
                  (Ope=Inf) or
                  (Ope=Sup) then
               Res:=new Simulation_Value(
                  Simulation_Time_Unit_Array_Boolean);
            else
               Res:=new Simulation_Value(
                  Simulation_Time_Unit_Array_Integer);
            end if;


            for I in Time_Unit_Range loop

               if Rvalue.Ptype=Simulation_Time_Unit_Array_Integer then

                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        Boolean_Time_Unit_Table_Value(I), Rvalue.
                        Boolean_Time_Unit_Table_Value(I), Ope);
                  else
                     Res.Integer_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        Integer_Time_Unit_Table_Value(I), Rvalue.
                        Integer_Time_Unit_Table_Value(I), Ope);
                  end if;

               else

                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        Boolean_Time_Unit_Table_Value(I), Rvalue.
                        Boolean_Value, Ope);
                  else
                     Res.Integer_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        Integer_Time_Unit_Table_Value(I), Rvalue.
                        Integer_Value, Ope);
                  end if;

               end if;

            end loop;





         when Simulation_Time_Unit_Array_Double =>


            if (Ope=Equal) or
                  (Ope=Not_Equal) or
                  (Ope=Equal_Less) or
                  (Ope=Equal_Greater) or
                  (Ope=Inf) or
                  (Ope=Sup) then
               Res:=new Simulation_Value(
                  Simulation_Time_Unit_Array_Boolean);
            else
               Res:=new Simulation_Value(
                  Simulation_Time_Unit_Array_Double);
            end if;

            for I in Time_Unit_Range loop

               if Rvalue.Ptype=Simulation_Time_Unit_Array_Double then
                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        Boolean_Time_Unit_Table_Value(I), Rvalue.
                        Boolean_Time_Unit_Table_Value(I),Ope);
                  else
                     Res.Double_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        Double_Time_Unit_Table_Value(I), Rvalue.
                        Double_Time_Unit_Table_Value(
                           I),Ope);
                  end if;
               else
                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        Boolean_Time_Unit_Table_Value(I), Rvalue.
                        Boolean_Value, Ope);
                  else
                     Res.Double_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        Double_Time_Unit_Table_Value(I), Rvalue.
                        Double_Value, Ope);
                  end if;
               end if;

            end loop;





         when Simulation_Time_Unit_Array_Boolean =>


            Res:=new Simulation_Value(
               Simulation_Time_Unit_Array_Boolean);

            for I in Time_Unit_Range loop

               if Rvalue.Ptype=Simulation_Time_Unit_Array_Boolean then
                  Res.Boolean_Time_Unit_Table_Value(I):=Apply_Operator(
                     Lvalue.Boolean_Time_Unit_Table_Value(I),
                     Rvalue.Boolean_Time_Unit_Table_Value(I),Ope);
               else
                  Res.Boolean_Time_Unit_Table_Value(I):=Apply_Operator(
                     Lvalue.
                     Boolean_Time_Unit_Table_Value(I),
                     Rvalue.Boolean_Value, Ope);
               end if;

            end loop;





         when Simulation_Time_Unit_Array_String =>


            if (Ope=Equal) or
                  (Ope=Not_Equal) or
                  (Ope=Equal_Less) or
                  (Ope=Equal_Greater) or
                  (Ope=Inf) or
                  (Ope=Sup) then
               Res:=new Simulation_Value(
                  Simulation_Time_Unit_Array_Boolean);
            else
               Res:=new Simulation_Value(
                  Simulation_Time_Unit_Array_String);
            end if;

            for I in Time_Unit_Range loop

               if Rvalue.Ptype=Simulation_Time_Unit_Array_String then
                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        Boolean_Time_Unit_Table_Value(I), Rvalue.
                        Boolean_Time_Unit_Table_Value(I),Ope);
                  else
                     Res.String_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        String_Time_Unit_Table_Value(I), Rvalue.
                        String_Time_Unit_Table_Value(
                           I),Ope);
                  end if;
               else
                  if (Ope=Equal) or
                        (Ope=Not_Equal) or
                        (Ope=Equal_Less) or
                        (Ope=Equal_Greater) or
                        (Ope=Inf) or
                        (Ope=Sup) then
                     Res.Boolean_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        Boolean_Time_Unit_Table_Value(I), Rvalue.
                        Boolean_Value, Ope);
                  else
                     Res.String_Time_Unit_Table_Value(I):=Apply_Operator(
                        Lvalue.
                        String_Time_Unit_Table_Value(I), Rvalue.
                        String_Value, Ope);
                  end if;
               end if;

            end loop;





      end case;

      return Res;

   end Compute_Binary_Value;





   function Apply_Operator (
         Value : Boolean; 
         Ope   : Operator ) 
     return Boolean is 
   begin

      if Ope=Logic_Not then
         return not Value;
      end if;

      Raise_Exception(Type_Error'Identity,
         To_String(Lb_Uncompatible_Type_Error(Current_Language) &
            Lb_Colon & To_Unbounded_String(Value'Img) &
            Lb_Comma &
            To_Unbounded_String(Ope'Img)) );
   end Apply_Operator;


   function Apply_Operator (
         Value : Integer; 
         Ope   : Operator ) 
     return Integer is 
   begin

      if Ope=Abs_Operator then
         return abs Value;
      end if;

      Raise_Exception(Type_Error'Identity,
         To_String(Lb_Uncompatible_Type_Error(Current_Language) &
            Lb_Colon & To_Unbounded_String(Value'Img) &
            Lb_Comma &
            To_Unbounded_String(Ope'Img)) );
   end Apply_Operator;


   function Apply_Operator (
         Value : Double;  
         Ope   : Operator ) 
     return Double is 
   begin

      if Ope=Abs_Operator then
         return abs Value;
      end if;

      Raise_Exception(Type_Error'Identity,
         To_String(Lb_Uncompatible_Type_Error(Current_Language) &
            Lb_Colon & To_Unbounded_String(Value'Img) &
            Lb_Comma &
            To_Unbounded_String(Ope'Img)) );
   end Apply_Operator;


   function Apply_Operator (
         Value : Unbounded_String;     
         V     : Variables_Table_Type; 
         Ope   : Operator              ) 
     return Integer is 

      Var : Variables_Range;  


   begin

      if Ope=Get_Task_Index then
         Var:=Find_Variable(V,
            To_Unbounded_String("tasks.name"));
         for I in String_Table'range loop
            if V.Entries(Var).Simulation.
                  String_Table_Value(I) = Value
                  then
               return Integer(I);
            end if;
         end loop;
         Raise_Exception(Statement_Error'Identity,
            To_String(Lb_Task_Not_Found(Current_Language) &
               Lb_Colon & Value &
               Lb_Comma &
               To_Unbounded_String(Ope'Img)) );
      end if;

      if Ope=Get_Buffer_Index then
         Var:=Find_Variable(V,
            To_Unbounded_String("buffers.name"));
         for I in String_Table'range loop
            if V.Entries(Var).Simulation.
                  String_Table_Value(I) = Value
                  then
               return Integer(I);
            end if;
         end loop;
         Raise_Exception(Statement_Error'Identity,
            To_String(Lb_Buffer_Not_Found(Current_Language) &
               Lb_Colon & Value &
               Lb_Comma &
               To_Unbounded_String(Ope'Img)) );
      end if;


      if Ope=Get_Message_Index then
         Var:=Find_Variable(V,
            To_Unbounded_String("messages.name"));
         for I in String_Table'range loop
            if V.Entries(Var).Simulation.
                  String_Table_Value(I) = Value
                  then
               return Integer(I);
            end if;
         end loop;
         Raise_Exception(Statement_Error'Identity,
            To_String(Lb_Message_Not_Found(Current_Language) &
               Lb_Colon & Value &
               Lb_Comma &
               To_Unbounded_String(Ope'Img)) );
      end if;

      if Ope=Get_Resource_Index then
         Var:=Find_Variable(V,
            To_Unbounded_String("resources.name"));
         for I in String_Table'range loop
            if V.Entries(Var).Simulation.
                  String_Table_Value(I) = Value
                  then
               return Integer(I);
            end if;
         end loop;
         Raise_Exception(Statement_Error'Identity,
            To_String(Lb_Resource_Not_Found(Current_Language) &
               Lb_Colon & Value &
               Lb_Comma &
               To_Unbounded_String(Ope'Img)) );
      end if;

      Raise_Exception(Type_Error'Identity,
         To_String(Lb_Uncompatible_Type_Error(Current_Language) &
            Lb_Colon & Value &
            Lb_Comma &
            To_Unbounded_String(Ope'Img)) );
   end Apply_Operator;



   ---------------------------------------------------------









   function Value_Of (
         C : Binary_Expr;         
         V : Variables_Table_Type ) 
     return Simulation_Value_Ptr is 

      Res,  
      Lvalue,  
      Rvalue : Simulation_Value_Ptr;  


   begin

      Lvalue:=Value_Of(C.Lvalue.All, V);
      Rvalue:=Value_Of(C.Rvalue.All, V);


      -- First case : it's a logical expression : the two values 
      -- have to have the same type and the same size
      -- 
      -- Second case : the two values has the same type (boolean, integer, double) and the
      -- same size (scalar or vector)
      --
      -- Third case :  the two values has the same type but a data is scalar and
      -- the other is a vector
      --
      -- Last case : types are differents : it's an error !
      --

      if (Rvalue.Ptype/=Lvalue.Ptype)  and
            (
            (C.Ope = Equal)
            or (C.Ope = Not_Equal)
            or (C.Ope = Equal_Less)
            or (C.Ope = Equal_Greater)
            or (C.Ope = Logic_And)
            or (C.Ope = Logic_Or)
            or (C.Ope = Logic_Not)
            or (C.Ope = Inf)
            or (C.Ope = Sup)  )
            then
         Raise_Exception(Type_Error'Identity,
            To_String(Lb_Uncompatible_Type_Error(Current_Language) &
               Lb_Colon & To_Unbounded_String(C.Lvalue.All) &
               Lb_Comma &
               To_Unbounded_String(C.Rvalue.All)));
      end if;


      if (
            (Rvalue.Ptype=Lvalue.Ptype)  or



            ( (Lvalue.Ptype=Simulation_Array_Integer) and  (
                  Rvalue.Ptype=Simulation_Integer) ) or

            ( (Lvalue.Ptype=Simulation_Array_Double) and  (
                  Rvalue.Ptype=Simulation_Double) ) or

            ( (Lvalue.Ptype=Simulation_Array_Boolean) and  (
                  Rvalue.Ptype=Simulation_Boolean) ) or

            ( (Lvalue.Ptype=Simulation_Array_String) and  (
                  Rvalue.Ptype=Simulation_String) ) or

            ( (Lvalue.Ptype=Simulation_Integer) and  (Rvalue.
                  Ptype=Simulation_Array_Integer) )  or

            ( (Lvalue.Ptype=Simulation_Double) and  (Rvalue.
                  Ptype=Simulation_Array_Double) ) or

            ( (Lvalue.Ptype=Simulation_String) and  (Rvalue.
                  Ptype=Simulation_Array_String) ) or

            ( (Lvalue.Ptype=Simulation_Boolean) and  (Rvalue.
                  Ptype=Simulation_Array_Boolean)) or




            ( (Lvalue.Ptype=Simulation_Time_Unit_Array_Integer) and  (
                  Rvalue.Ptype=Simulation_Integer) ) or

            ( (Lvalue.Ptype=Simulation_Time_Unit_Array_Double) and  (
                  Rvalue.Ptype=Simulation_Double) ) or

            ( (Lvalue.Ptype=Simulation_Time_Unit_Array_Boolean) and  (
                  Rvalue.Ptype=Simulation_Boolean) ) or

            ( (Lvalue.Ptype=Simulation_Time_Unit_Array_String) and  (
                  Rvalue.Ptype=Simulation_String) ) or

            ( (Lvalue.Ptype=Simulation_Integer) and  (Rvalue.
                  Ptype=Simulation_Time_Unit_Array_Integer) )  or

            ( (Lvalue.Ptype=Simulation_Double) and  (Rvalue.
                  Ptype=Simulation_Time_Unit_Array_Double) ) or

            ( (Lvalue.Ptype=Simulation_String) and  (Rvalue.
                  Ptype=Simulation_Time_Unit_Array_String) ) or

            ( (Lvalue.Ptype=Simulation_Boolean) and  (Rvalue.
                  Ptype=Simulation_Time_Unit_Array_Boolean) )
            )

            then
         Res:=Compute_Binary_Value(Lvalue, Rvalue, C.Ope);

      else
         Raise_Exception(Type_Error'Identity,
            To_String(Lb_Uncompatible_Type_Error(Current_Language) &
               Lb_Colon & To_Unbounded_String(C.Lvalue.All) &
               Lb_Comma &
               To_Unbounded_String(C.Rvalue.All)));
      end if;


      return Res;

   end Value_Of;



   function Value_Of (
         C : Unary_Expr;          
         V : Variables_Table_Type ) 
     return Simulation_Value_Ptr is 

      Res,  
      Value : Simulation_Value_Ptr;  


   begin


      Value:=Value_Of(C.Value.All,V);


      if (Value.Ptype=Simulation_Array_String)
            or (Value.Ptype=Simulation_Time_Unit_Array_String)
            then
         Raise_Exception(Type_Error'Identity,
            To_String(Lb_Uncompatible_Type_Error(Current_Language) &
               Lb_Colon & Value.Ptype'Img &
               Lb_Comma &
               To_Unbounded_String(C.Ope'Img)) );
      end if;


      if (C.Ope=Max_To_Index) or (C.Ope=Min_To_Index)
            then
         return Value;
      end if;

      -- Apply operator from an unary expression
      --
      case Value.Ptype is

         when Simulation_String=>
            Res:=new Simulation_Value(
               Simulation_Integer);
            Res.Integer_Value:=Apply_Operator(Value.String_Value, V, C.
               Ope);

         when Simulation_Array_String =>
            null;
         when Simulation_Time_Unit_Array_String =>
            null;

         -- Nothing to compute in the cases of random type
         -- because value_of functions already do the job
         -- by returing integer values
         --
         when Simulation_Random =>
            null;

         when Simulation_Array_Random =>
            null;

         when Simulation_Time_Unit_Array_Random =>
            null;



         when Simulation_Time_Unit_Array_Integer =>
            Res:=new Simulation_Value(
               Simulation_Time_Unit_Array_Integer);
            for I in Time_Unit_Range loop
               Res.Integer_Time_Unit_Table_Value(I):=
                  Apply_Operator(Value.Integer_Time_Unit_Table_Value(I),
                  C.
                  Ope);
            end loop;


         when Simulation_Array_Integer =>
            Res:=new Simulation_Value(
               Simulation_Array_Integer);
            for I in Simulations_Range loop
               Res.Integer_Table_Value(I):=
                  Apply_Operator(Value.Integer_Table_Value(I), C.Ope);
            end loop;

         when Simulation_Integer =>
            Res:=new Simulation_Value(
               Simulation_Integer);
            Res.Integer_Value:=Apply_Operator(Value.Integer_Value, C.Ope);

         when Simulation_Time_Unit_Array_Double =>
            Res:=new Simulation_Value(
               Simulation_Time_Unit_Array_Double);
            for I in Time_Unit_Range loop
               Res.Double_Time_Unit_Table_Value(I):=
                  Apply_Operator(Value.Double_Time_Unit_Table_Value(I), C.
                  Ope);
            end loop;

         when Simulation_Array_Double =>
            Res:=new Simulation_Value(
               Simulation_Array_Double);
            for I in Simulations_Range loop
               Res.Double_Table_Value(I):=
                  Apply_Operator(Value.Double_Table_Value(I), C.Ope);
            end loop;

         when Simulation_Double =>
            Res:=new Simulation_Value(
               Simulation_Double);
            Res.Double_Value:=Apply_Operator(Value.Double_Value, C.Ope);

         when Simulation_Time_Unit_Array_Boolean =>
            Res:=new Simulation_Value(
               Simulation_Time_Unit_Array_Boolean);
            for I in Time_Unit_Range loop
               Res.Boolean_Time_Unit_Table_Value(I):=
                  Apply_Operator(Value.Boolean_Time_Unit_Table_Value(I),
                  C.
                  Ope);
            end loop;

         when Simulation_Array_Boolean =>
            Res:=new Simulation_Value(
               Simulation_Array_Boolean);
            for I in Simulations_Range loop
               Res.Boolean_Table_Value(I):=
                  Apply_Operator(Value.Boolean_Table_Value(I), C.Ope);
            end loop;

         when Simulation_Boolean =>
            Res:=new Simulation_Value(
               Simulation_Boolean);
            Res.Boolean_Value:=Apply_Operator(Value.Boolean_Value, C.Ope);
      end case;

      return Res;

   end Value_Of;

   function Value_Of (
         C : Constant_Expr;       
         V : Variables_Table_Type ) 
     return Simulation_Value_Ptr is 

   begin
      return Copy(C.Value);
   end Value_Of;


   function Value_Of (
         C : Variable_Expr;       
         V : Variables_Table_Type ) 
     return Simulation_Value_Ptr is 

      New_Val : Simulation_Value_Ptr;  
      Seed    : Generator;  

   begin
      for I in 0..V.Nb_Entries-1 loop
         if V.Entries(I).Variable.Name=C.Name
               then
            if   (V.Entries(I).Simulation.Ptype = Simulation_Random)
                  then
               Reset(Seed, V.Entries(I).Simulation.Random_Value);
               New_Val:=new Simulation_Value(Simulation_Integer);
               New_Val.Integer_Value:=
                  Apply_Random_Operator(Seed, V.Entries(I).Simulation.
                  Random_Law_Parameters);
               Save(Seed, V.Entries(I).Simulation.Random_Value);
               return New_Val;
            else
               return Copy(V.Entries(I).Simulation);
            end if;
         end if;
      end loop;

      Raise_Exception(Variable_Error'Identity,
         To_String(Lb_Undeclared_Identifier(Current_Language) & Lb_Colon &
            C.Name));
   end Value_Of;



   function Value_Of (
         C : Array_Variable_Expr; 
         V : Variables_Table_Type ) 
     return Simulation_Value_Ptr is 

      New_Val : Simulation_Value_Ptr;  
      Index   : Simulation_Value_Ptr;  
      Seed    : Generator;  

   begin
      for I in 0..V.Nb_Entries-1 loop
         if V.Entries(I).Variable.Name=C.Name
               then

            -- Two cases : return the all task array table or
            --          if an index is provided, return
            --          a scalar data
            --

            if (C.Array_Index/=null)


                  -- indexed expression : return scalar data
                  --
                  then

               Index:=Value_Of(C.Array_Index.All,V);
               if Index.Ptype/=Simulation_Integer
                     then
                  Raise_Exception(Type_Error'Identity, To_String(
                        Lb_Index_Error(Current_Language) & Lb_Colon & C.
                        Name));
               end if;

               if C.Variable_Type=Simulation_Array_Integer then
                  New_Val:=new Simulation_Value(Simulation_Integer);
                  New_Val.Integer_Value:=V.Entries(I).Simulation.
                     Integer_Table_Value(Simulations_Range(Index.
                        Integer_Value));
               end if;
               if C.Variable_Type=Simulation_Array_Random then
                  Reset(Seed, V.Entries(I).Simulation.Random_Table_Value(
                        Simulations_Range(Index.Integer_Value)));
                  New_Val:=new Simulation_Value(Simulation_Integer);
                  New_Val.Integer_Value:=
                     Apply_Random_Operator(Seed, V.Entries(I).Simulation.
                     Random_Table_Law_Parameters);
                  Save(Seed, V.Entries(I).Simulation.Random_Table_Value(
                        Simulations_Range(Index.Integer_Value)));
               end if;
               if C.Variable_Type=Simulation_Array_Double then
                  New_Val:=new Simulation_Value(Simulation_Double);
                  New_Val.Double_Value:=V.Entries(I).Simulation.
                     Double_Table_Value(Simulations_Range(Index.
                        Integer_Value));
               end if;
               if C.Variable_Type=Simulation_Array_Boolean then
                  New_Val:=new Simulation_Value(Simulation_Boolean);
                  New_Val.Boolean_Value:=V.Entries(I).Simulation.
                     Boolean_Table_Value(Simulations_Range(Index.
                        Integer_Value));
               end if;
               if C.Variable_Type=Simulation_Array_String then
                  New_Val:=new Simulation_Value(Simulation_String);
                  New_Val.String_Value:=V.Entries(I).Simulation.
                     String_Table_Value(Simulations_Range(Index.
                        Integer_Value));
               end if;
               if C.Variable_Type=Simulation_Time_Unit_Array_Integer then
                  New_Val:=new Simulation_Value(Simulation_Integer);
                  New_Val.Integer_Value:=V.Entries(I).Simulation.
                     Integer_Time_Unit_Table_Value(Time_Unit_Range(Index.
                        Integer_Value));
               end if;
               if C.Variable_Type=Simulation_Time_Unit_Array_Random then
                  Reset(Seed, V.Entries(I).
                     Simulation.Random_Time_Unit_Table_Value(
                        Time_Unit_Range(Index.Integer_Value)));
                  New_Val:=new Simulation_Value(Simulation_Integer);
                  New_Val.Integer_Value:=
                     Apply_Random_Operator(Seed, V.Entries(I).Simulation.
                     Random_Time_Unit_Table_Law_Parameters);
                  Save(Seed, V.Entries(I).Simulation.
                     Random_Time_Unit_Table_Value(
                        Time_Unit_Range(Index.Integer_Value)));
               end if;
               if C.Variable_Type=Simulation_Time_Unit_Array_Double then
                  New_Val:=new Simulation_Value(Simulation_Double);
                  New_Val.Double_Value:=V.Entries(I).Simulation.
                     Double_Time_Unit_Table_Value(Time_Unit_Range(Index.
                        Integer_Value));
               end if;
               if C.Variable_Type=Simulation_Time_Unit_Array_Boolean then
                  New_Val:=new Simulation_Value(Simulation_Boolean);
                  New_Val.Boolean_Value:=V.Entries(I).Simulation.
                     Boolean_Time_Unit_Table_Value(Time_Unit_Range(Index.
                        Integer_Value));
               end if;
               if C.Variable_Type=Simulation_Time_Unit_Array_String then
                  New_Val:=new Simulation_Value(Simulation_String);
                  New_Val.String_Value:=V.Entries(I).Simulation.
                     String_Time_Unit_Table_Value(Time_Unit_Range(Index.
                        Integer_Value));
               end if;

               return New_Val;


            else
               -- Array tables.
               --

               if C.Variable_Type=Simulation_Array_Random then
                  New_Val:=new Simulation_Value(Simulation_Array_Integer);
                  for J in Simulations_Range loop
                     Reset(Seed, V.Entries(I).
                        Simulation.Random_Table_Value(J));
                     New_Val.Integer_Table_Value(J):=
                        Apply_Random_Operator(Seed, V.Entries(I).
                        Simulation.Random_Table_Law_Parameters);
                     Save(Seed, V.Entries(I).
                        Simulation.Random_Table_Value(J));
                  end loop;
               end if;

               if C.Variable_Type=Simulation_Array_Integer then
                  New_Val:=new Simulation_Value(
                     Simulation_Array_Integer);
                  New_Val.Integer_Table_Value:=V.Entries(I).Simulation.
                     Integer_Table_Value;
               end if;

               if C.Variable_Type=Simulation_Array_Double then
                  New_Val:=new Simulation_Value(
                     Simulation_Array_Double);
                  New_Val.Double_Table_Value:=V.Entries(I).Simulation.
                     Double_Table_Value;
               end if;

               if C.Variable_Type=Simulation_Array_Boolean then
                  New_Val:=new Simulation_Value(
                     Simulation_Array_Boolean);
                  New_Val.Boolean_Table_Value:=V.Entries(I).Simulation.
                     Boolean_Table_Value;
               end if;

               if C.Variable_Type=Simulation_Array_String then
                  New_Val:=new Simulation_Value(
                     Simulation_Array_String);
                  New_Val.String_Table_Value:=V.Entries(I).Simulation.
                     String_Table_Value;
               end if;

               if C.Variable_Type=Simulation_Time_Unit_Array_Random then
                  New_Val:=new Simulation_Value(
                     Simulation_Time_Unit_Array_Integer);
                  for J in Time_Unit_Range loop
                     Reset(Seed, V.Entries(I).
                        Simulation.Random_Time_Unit_Table_Value(J));
                     New_Val.Integer_Time_Unit_Table_Value(J):=
                        Apply_Random_Operator(Seed, V.Entries(I).
                        Simulation.Random_Time_Unit_Table_Law_Parameters);
                     Save(Seed, V.Entries(I).
                        Simulation.Random_Time_Unit_Table_Value(J));
                  end loop;
               end if;

               if C.Variable_Type=Simulation_Time_Unit_Array_Integer then
                  New_Val:=new Simulation_Value(
                     Simulation_Time_Unit_Array_Integer);
                  New_Val.Integer_Time_Unit_Table_Value:=V.Entries(I).
                     Simulation.
                     Integer_Time_Unit_Table_Value;
               end if;

               if C.Variable_Type=Simulation_Time_Unit_Array_Double then
                  New_Val:=new Simulation_Value(
                     Simulation_Time_Unit_Array_Double);
                  New_Val.Double_Time_Unit_Table_Value:=V.Entries(I).
                     Simulation.
                     Double_Time_Unit_Table_Value;
               end if;

               if C.Variable_Type=Simulation_Time_Unit_Array_Boolean then
                  New_Val:=new Simulation_Value(
                     Simulation_Time_Unit_Array_Boolean);
                  New_Val.Boolean_Time_Unit_Table_Value:=V.Entries(I).
                     Simulation.
                     Boolean_Time_Unit_Table_Value;
               end if;

               if C.Variable_Type=Simulation_Time_Unit_Array_String then
                  New_Val:=new Simulation_Value(
                     Simulation_Time_Unit_Array_String);
                  New_Val.String_Time_Unit_Table_Value:=V.Entries(I).
                     Simulation.
                     String_Time_Unit_Table_Value;
               end if;


               return New_Val;
            end if;


         end if;

      end loop;


      Raise_Exception(Variable_Error'Identity,
         To_String(Lb_Undeclared_Identifier(Current_Language) & Lb_Colon &
            C.Name));
   end Value_Of;








end Expressions;






