



with Text_Io;
use Text_Io;
with Config;
use Config;
with Scheduler_Lex;
use Scheduler_Lex;
with Scheduler_Dfa;
use Scheduler_Dfa;
with Scheduler_Goto; 
use Scheduler_Goto;
with Scheduler_Tokens; 
use Scheduler_Tokens;
with Scheduler_Shift_Reduce;
use Scheduler_Shift_Reduce;
with Ada.Numerics.Aux;
use Ada.Numerics.Aux;
with Translate;          
use Translate;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;
with Ada.Exceptions;
use Ada.Exceptions;
with Simulations;
use Simulations;
with Parameters;
use Parameters;
use Parameters.User_Defined_Parameters_Table_Package;
with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

package body Parser is

  
        -- The langague is block oriented : the constant below
        -- given the maximum depth level of block
        --
        Max_Block_Level : constant Natural := 10000;


        -- These variables store statements of inner blocks of code
        --
        Else_Inner_Statement_Ptr   : Array (0..Max_Block_Level) of Statement_Ptr;
        Else_Inner_Statement_Index : Natural :=0;
        Then_Inner_Statement_Ptr   : Array (0..Max_Block_Level) of Statement_Ptr;
        Then_Inner_Statement_Index : Natural :=0;
        Loop_Inner_Statement_Ptr   : Array (0..Max_Block_Level) of Statement_Ptr;
        Loop_Inner_Statement_Index : Natural :=0;


        -- Variables needed to parse a "if" or a "for" statement
        --
        New_If     : If_Statement_Ptr;
        Old_Stat   : Statement_Ptr;
        New_Stat   : Statement_Ptr;
        New_for    : For_Statement_Ptr;
        New_While  : While_Statement_Ptr;


	procedure Yyerror(S: String) is
	begin
                Raise_Exception(Expressions.Syntax_Error'Identity, "token """ & To_String(
                        To_Unbounded_String(Yytext) & """" & Lb_Comma & " Line" & positive'image(Lines) & Lb_Comma & To_Unbounded_String(S) ));
	end Yyerror;



	-- Store the scheduler behavior step
	-- Used to Check is some Statements can be
	-- declared in the Current_Section
	--
	Current_Section : Sections;


	-- Current Pointers to the parsed code
	-- There is one Pointers per section (table of pointers)
	--
	Current_Statement_Pointer : Section_Pointers;



	-- Procedure used to display content of each
	-- section (Statements and Expressions)
	--
	package Sections_Io is new Text_Io.Enumeration_Io(Sections);
	use Sections_Io;
	
	procedure Put(S : in Section_Pointers) is 
	
	begin 
		for I in Sections'range loop
			if S(I)/=null 
				then
					Put(" Section : ");
					Put(I); New_Line;	
					Recursive_Put(S(I));
					New_Line;
					New_Line;
			end if;
	    	end loop;
	end Put;



        procedure Init_Parser(My_Tasks : in Tasks_set) is

   
	New_Var    	  : Variable_Record_Ptr;
	New_Expr_Array 	  : Array_Variable_Expr_Ptr;
	New_Expr_Variable : Variable_Expr_Ptr;
        Do_Insert         : Boolean;
        A_Task            : Task_Ptr;
        Iterator          : Tasks_Iterator;     

         begin

                Else_Inner_Statement_Index:=0;
                Then_Inner_Statement_Index:=0;
                Loop_Inner_Statement_Index:=0;

		Initialize(Variables_Table);
		Initialize(Sets_Table);


		for I in Sections'range loop
			Root_Statement_Pointer(I):=null;
		end loop;

                Scheduler_Lex.Lines:=1;


                ------------------------------------------------------
                --              common variables
                ------------------------------------------------------


                -- "previously_elected"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("previously_elected");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "nb_tasks"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("nb_tasks");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "nb_processors"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("nb_processors");
                New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "nb_messages"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("nb_messages");
                New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "nb_buffers"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("nb_buffers");
                New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "nb_resources"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("nb_resources");
                New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "simulation_time"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("simulation_time");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                ------------------------------------------------------
                --              Buffer variables
                ------------------------------------------------------


                -- "name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("buffers.name");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "max_size"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("buffers.max_size");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "processor_name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("buffers.processor_name");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "users.time"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("buffers.users.time");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "users.size"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("buffers.users.size");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "users.task_name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("buffers.users.task_name");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);

            
                ------------------------------------------------------
                --              Message variables
                ------------------------------------------------------


                -- "jitter"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("messages.jitter");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);



                -- "delay"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("messages.delay");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("messages.name");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "deadline"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("messages.deadline");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "period"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("messages.period");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "size"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("messages.size");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "users.time"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("messages.users.time");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "users.type"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("messages.users.type");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "users.task_name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("messages.users.task_name");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);



                ------------------------------------------------------
                --              Event variables
                ------------------------------------------------------


                -- "events.time"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("events.time");
                New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "events.processor_name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("events.processor_name");
                New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "events.type"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("events.type");
                New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "events.task_name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("events.task_name");
                New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "events.buffer_name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("events.buffer_name");
                New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "events.resource_name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("events.resource_name");
                New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "events.message_name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("events.message_name");
                New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                ------------------------------------------------------
                --              Resource variables
                ------------------------------------------------------



                -- "initial_state"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("resources.initial_state");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "processor_name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("resources.processor_name");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "protocol"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("resources.protocol");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("resources.name");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "users.task_name"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_String;
                New_Expr_Variable.Name:=To_Unbounded_String("resources.users.task_name");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "users.start_time"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("resources.users.start_time");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);


                -- "users.end_time"
                --
                New_Var:= new Variable_record;
                New_Expr_Variable := new Variable_Expr;
                New_Expr_Variable.Variable_Type:=Simulation_Integer;
                New_Expr_Variable.Name:=To_Unbounded_String("resources.users.end_time");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Variable);
                Add(Variables_Table, New_Var);



                ------------------------------------------------------
                --              Task variables
                ------------------------------------------------------


		-- "priority"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer;
		New_Expr_Array.Name:=To_Unbounded_String("tasks.priority");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
	
	
		-- "deadline"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer;
       	 	New_Expr_Array.Name:=To_Unbounded_String("tasks.deadline");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
		

		-- "start_time"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer;
        	New_Expr_Array.Name:=To_Unbounded_String("tasks.start_time");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
	

		-- "capacity"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer;
       	 	New_Expr_Array.Name:=To_Unbounded_String("tasks.capacity");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);


		-- "rest_of_capacity"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer;
        	New_Expr_Array.Name:=To_Unbounded_String("tasks.rest_of_capacity");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);


		-- "activation_number"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer;
        	New_Expr_Array.Name:=To_Unbounded_String("tasks.activation_number");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
		

		-- "used_cpu"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer;
        	New_Expr_Array.Name:=To_Unbounded_String("tasks.used_cpu");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
		

		-- "period"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer;
        	New_Expr_Array.Name:=To_Unbounded_String("tasks.period");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);
		

                -- "ready"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Boolean;
       	 	New_Expr_Array.Name:=To_Unbounded_String("tasks.ready");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);


                -- "processor_name"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_String;
       	 	New_Expr_Array.Name:=To_Unbounded_String("tasks.processor_name");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);


                -- "blocking_time"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer;
       	 	New_Expr_Array.Name:=To_Unbounded_String("tasks.blocking_time");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);


                -- "name"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_String;
       	 	New_Expr_Array.Name:=To_Unbounded_String("tasks.name");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);


                -- "jitter"
		--
		New_Var:= new Variable_record;
                New_Expr_Array := new Array_Variable_Expr;
		New_Expr_Array.Variable_Type:=Simulation_Array_Integer;
       	 	New_Expr_Array.Name:=To_Unbounded_String("tasks.jitter");
       	 	New_Var.Variable:=variable_expr_Ptr(New_Expr_Array);
		Add(Variables_Table, New_Var);


		-- Task' user defined parameters
		--
		-- Find user defined parameters : scan all task
		-- Variables and add them
		--

		if not Is_Empty(My_Tasks) 
			then
                                Reset_Iterator(My_Tasks, Iterator);

         			loop
                                        Current_Element(My_Tasks, A_Task, Iterator);

                                        exit when Is_Last_Element(My_Tasks, Iterator) = True;

		 			for I in 0..A_Task.Param.Nb_Entries-1 loop
  
						-- Is the Variable already exists ?
                				--
						Do_Insert:=true;
                				for J in 0..Variables_Table.Nb_Entries-1 loop
                                                        if Variables_Table.Entries(J).Variable.Name=
                                                                To_Unbounded_String("tasks.") & A_Task.Param.Entries(i).Name
                                                                then Do_Insert:=False;
                        				end if;
                				end loop;

						if Do_Insert 
							then
								New_Var:= new Variable_record;
						        	New_Expr_Array := new 
									Array_Variable_Expr;
						        	if A_Task.Param.Entries(I).Ptype=Integer_Parameter
									then
										New_Expr_Array.Variable_Type:=Simulation_Array_Integer;
								end if;
						        	if A_Task.Param.Entries(I).Ptype=Double_Parameter
									then
										New_Expr_Array.Variable_Type:=Simulation_Array_Double;
								end if;
						        	if A_Task.Param.Entries(I).Ptype=Boolean_Parameter
									then
										New_Expr_Array.Variable_Type:=Simulation_Array_Boolean;
								end if;
						        	if A_Task.Param.Entries(I).Ptype=String_Parameter
									then
										New_Expr_Array.Variable_Type:=Simulation_Array_String;
								end if;
	
                                                                New_Expr_Array.Name:=To_Unbounded_String("tasks.") & A_Task.Param.Entries(I).Name;
                                                                New_Var.Variable:=Variable_Expr_Ptr(New_Expr_Array);
								Add(Variables_Table, New_Var);
						end if;
	
					end loop;

                                        Next_Element(My_Tasks, Iterator);
				end loop;
			end if;




      ---------------------------------------------------------------
      -- Initialize simulation data : provide initial values for
      --  all variables 
      ---------------------------------------------------------------


      -- Allocate simulation data
      --  	
      for I in 0..Variables_Table.Nb_Entries-1 loop

         Variables_Table.Entries(I).Simulation:=new
            Simulation_Value(
            Variable_Expr_Ptr(Variables_Table.
               Entries(
                  I).
               Variable).Variable_Type);

      end loop;


            


 end Init_Parser;





procedure YYParse is

   -- Rename User Defined Packages to Internal Names.
    package yy_goto_tables         renames
      Scheduler_Goto;
    package yy_shift_reduce_tables renames
      Scheduler_Shift_Reduce;
    package yy_tokens              renames
      Scheduler_Tokens;

   use yy_tokens, yy_goto_tables, yy_shift_reduce_tables;

   procedure yyerrok;
   procedure yyclearin;


   package yy is

       -- the size of the value and state stacks
       stack_size : constant Natural := 300;

       -- subtype rule         is natural;
       subtype parse_state  is natural;
       -- subtype nonterminal  is integer;

       -- encryption constants
       default           : constant := -1;
       first_shift_entry : constant :=  0;
       accept_code       : constant := -3001;
       error_code        : constant := -3000;

       -- stack data used by the parser
       tos                : natural := 0;
       value_stack        : array(0..stack_size) of yy_tokens.yystype;
       state_stack        : array(0..stack_size) of parse_state;

       -- current input symbol and action the parser is on
       action             : integer;
       rule_id            : rule;
       input_symbol       : yy_tokens.token;


       -- error recovery flag
       error_flag : natural := 0;
          -- indicates  3 - (number of valid shifts after an error occurs)

       look_ahead : boolean := true;
       index      : integer;

       -- Is Debugging option on or off
        DEBUG : constant boolean := FALSE;

    end yy;


    function goto_state
      (state : yy.parse_state;
       sym   : nonterminal) return yy.parse_state;

    function parse_action
      (state : yy.parse_state;
       t     : yy_tokens.token) return integer;

    pragma inline(goto_state, parse_action);


    function goto_state(state : yy.parse_state;
                        sym   : nonterminal) return yy.parse_state is
        index : integer;
    begin
        index := goto_offset(state);
        while  integer(goto_matrix(index).nonterm) /= sym loop
            index := index + 1;
        end loop;
        return integer(goto_matrix(index).newstate);
    end goto_state;


    function parse_action(state : yy.parse_state;
                          t     : yy_tokens.token) return integer is
        index      : integer;
        tok_pos    : integer;
        default    : constant integer := -1;
    begin
        tok_pos := yy_tokens.token'pos(t);
        index   := shift_reduce_offset(state);
        while integer(shift_reduce_matrix(index).t) /= tok_pos and then
              integer(shift_reduce_matrix(index).t) /= default
        loop
            index := index + 1;
        end loop;
        return integer(shift_reduce_matrix(index).act);
    end parse_action;

-- error recovery stuff

    procedure handle_error is
      temp_action : integer;
    begin

      if yy.error_flag = 3 then -- no shift yet, clobber input.
      if yy.debug then
          text_io.put_line("Ayacc.YYParse: Error Recovery Clobbers " &
                   yy_tokens.token'image(yy.input_symbol));
      end if;
        if yy.input_symbol = yy_tokens.end_of_input then  -- don't discard,
        if yy.debug then
            text_io.put_line("Ayacc.YYParse: Can't discard END_OF_INPUT, quiting...");
        end if;
        raise yy_tokens.syntax_error;
        end if;

            yy.look_ahead := true;   -- get next token
        return;                  -- and try again...
    end if;

    if yy.error_flag = 0 then -- brand new error
        yyerror("Syntax Error");
    end if;

    yy.error_flag := 3;

    -- find state on stack where error is a valid shift --

    if yy.debug then
        text_io.put_line("Ayacc.YYParse: Looking for state with error as valid shift");
    end if;

    loop
        if yy.debug then
          text_io.put_line("Ayacc.YYParse: Examining State " &
               yy.parse_state'image(yy.state_stack(yy.tos)));
        end if;
        temp_action := parse_action(yy.state_stack(yy.tos), error);

            if temp_action >= yy.first_shift_entry then
                if yy.tos = yy.stack_size then
                    text_io.put_line(" Stack size exceeded on state_stack");
                    raise yy_Tokens.syntax_error;
                end if;
                yy.tos := yy.tos + 1;
                yy.state_stack(yy.tos) := temp_action;
                exit;
            end if;

        Decrement_Stack_Pointer :
        begin
          yy.tos := yy.tos - 1;
        exception
          when Constraint_Error =>
            yy.tos := 0;
        end Decrement_Stack_Pointer;

        if yy.tos = 0 then
          if yy.debug then
            text_io.put_line("Ayacc.YYParse: Error recovery popped entire stack, aborting...");
          end if;
          raise yy_tokens.syntax_error;
        end if;
    end loop;

    if yy.debug then
        text_io.put_line("Ayacc.YYParse: Shifted error token in state " &
              yy.parse_state'image(yy.state_stack(yy.tos)));
    end if;

    end handle_error;

   -- print debugging information for a shift operation
   procedure shift_debug(state_id: yy.parse_state; lexeme: yy_tokens.token) is
   begin
       text_io.put_line("Ayacc.YYParse: Shift "& yy.parse_state'image(state_id)&" on input symbol "&
               yy_tokens.token'image(lexeme) );
   end;

   -- print debugging information for a reduce operation
   procedure reduce_debug(rule_id: rule; state_id: yy.parse_state) is
   begin
       text_io.put_line("Ayacc.YYParse: Reduce by rule "&rule'image(rule_id)&" goto state "&
               yy.parse_state'image(state_id));
   end;

   -- make the parser believe that 3 valid shifts have occured.
   -- used for error recovery.
   procedure yyerrok is
   begin
       yy.error_flag := 0;
   end yyerrok;

   -- called to clear input symbol that caused an error.
   procedure yyclearin is
   begin
       -- yy.input_symbol := yylex;
       yy.look_ahead := true;
   end yyclearin;


begin
    -- initialize by pushing state 0 and getting the first input symbol
    yy.state_stack(yy.tos) := 0;


    loop

        yy.index := shift_reduce_offset(yy.state_stack(yy.tos));
        if integer(shift_reduce_matrix(yy.index).t) = yy.default then
            yy.action := integer(shift_reduce_matrix(yy.index).act);
        else
            if yy.look_ahead then
                yy.look_ahead   := false;

                yy.input_symbol := yylex;
            end if;
            yy.action :=
             parse_action(yy.state_stack(yy.tos), yy.input_symbol);
        end if;


        if yy.action >= yy.first_shift_entry then  -- SHIFT

            if yy.debug then
                shift_debug(yy.action, yy.input_symbol);
            end if;

            -- Enter new state
            if yy.tos = yy.stack_size then
                text_io.put_line(" Stack size exceeded on state_stack");
                raise yy_Tokens.syntax_error;
            end if;
            yy.tos := yy.tos + 1;
            yy.state_stack(yy.tos) := yy.action;
              yy.value_stack(yy.tos) := yylval;

        if yy.error_flag > 0 then  -- indicate a valid shift
            yy.error_flag := yy.error_flag - 1;
        end if;

            -- Advance lookahead
            yy.look_ahead := true;

        elsif yy.action = yy.error_code then       -- ERROR

            handle_error;

        elsif yy.action = yy.accept_code then
            if yy.debug then
                text_io.put_line("Ayacc.YYParse: Accepting Grammar...");
            end if;
            exit;

        else -- Reduce Action

            -- Convert action into a rule
            yy.rule_id  := -1 * yy.action;

            -- Execute User Action
            -- user_action(yy.rule_id);


                case yy.rule_id is

when  2 =>
--#line  68

		Current_Section:=Start;
	

when  5 =>
--#line  75

		Current_Section:=Priority;
	

when  8 =>
--#line  82

		Current_Section:=Election;
	

when  11 =>
--#line  89

		Current_Section:=Activation;
	

when  14 =>
--#line  96

		Current_Section:=Check_Resource;
	

when  17 =>
--#line  103

		Current_Section:=Release_Resource;
	

when  20 =>
--#line  110

		Current_Section:=Allocate_Resource;
	

when  23 =>
--#line  117

                Current_Section:=Gather_Event_Analyzer;
	

when  26 =>
--#line  124

                Current_Section:=Display_Event_Analyzer;
	

when  29 =>
--#line  134
	
        if 
yy.value_stack(yy.tos).Stats/=null then
		if (Root_Statement_Pointer(Current_Section)=null)
                then
                	Root_Statement_Pointer(Current_Section):=
yy.value_stack(yy.tos).Stats;
                        Current_Statement_Pointer(Current_Section):=Root_Statement_Pointer(Current_Section);
                else
                        Current_Statement_Pointer(Current_Section).Next_Statement:=
yy.value_stack(yy.tos).Stats;
                        Current_Statement_Pointer(Current_Section):=
yy.value_stack(yy.tos).Stats;
                end if;
        end if;

        

when  31 =>
--#line  153

                
yy.value_stack(yy.tos).Stats.Next_Statement:=Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index);
                Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index):=
yy.value_stack(yy.tos).Stats;
        

when  32 =>
--#line  158

                Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index+1;
                Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index):=
yy.value_stack(yy.tos).Stats;
                Loop_Inner_Statement_ptr(Loop_Inner_Statement_Index).Next_Statement:=null;
        

when  33 =>
--#line  166

                
yy.value_stack(yy.tos).Stats.Next_Statement:=Then_Inner_Statement_Ptr(Then_Inner_Statement_Index);
                Then_Inner_Statement_Ptr(Then_Inner_Statement_Index):=
yy.value_stack(yy.tos).Stats;
        

when  34 =>
--#line  171

                Then_Inner_Statement_Index:=Then_Inner_Statement_Index+1;
                Then_Inner_Statement_Ptr(Then_Inner_Statement_Index):=
yy.value_stack(yy.tos).Stats;
                Then_Inner_Statement_ptr(Then_Inner_Statement_Index).Next_Statement:=null;
        

when  35 =>
--#line  180

                
yy.value_stack(yy.tos).Stats.Next_Statement:=Else_Inner_Statement_Ptr(Else_Inner_Statement_Index);
                Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=
yy.value_stack(yy.tos).Stats;
        

when  36 =>
--#line  185

                Else_Inner_Statement_Index:=Else_Inner_Statement_Index+1;
                Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=
yy.value_stack(yy.tos).Stats;
                Else_Inner_Statement_ptr(Else_Inner_Statement_Index).Next_Statement:=null;
        

when  37 =>
--#line  195

	declare
		New_Put : Put_Statement_Ptr;
	begin

		-- Firstly, we Check if the Variable exists
		--
		Check_Variable_Declaration(Variables_Table, 
yy.value_stack(yy.tos-4).String_Value);

		New_Put:= new Put_Statement;
                New_Put.Identifier:=
yy.value_stack(yy.tos-4).String_Value;
                New_Put.Put_From:=
yy.value_stack(yy.tos-3).Integer_Value;
                New_Put.Put_To:=
yy.value_stack(yy.tos-2).Integer_Value;
		New_Put.Line_Number:=Scheduler_Lex.Lines;
		
yyval.Stats:=Statement_Ptr(New_Put);

	end;
	

when  38 =>
--#line  216

	declare
                New_Affectation : Affectation_Statement_Ptr;
		New_Var      	: Variable_Record_Ptr;
		New_Array_Expr 	: Array_Variable_Expr_Ptr;
                New_Var_Expr    : Variable_Expr_Ptr;
               
        
	begin

		-- Is the Variable already exists ?
		--
		for I in 0..Variables_Table.Nb_Entries-1 loop
			if (Variables_Table.Entries(i).Variable.Name=
yy.value_stack(yy.tos-4).String_Value) 
				then
					Raise_Exception(Expressions.Syntax_Error'Identity, To_String(
                         
yy.value_stack(yy.tos-4).String_Value & Lb_Comma & Lb_Identifier_Already_Declared(Current_Language) ));
			end if;
		end loop;



		-- Create the Variable (Array or not)
		--
		New_Var:= new Variable_Record;

                if (
yy.value_stack(yy.tos-2).Variable_Type_Value=Simulation_Array_Double)
                	or (
yy.value_stack(yy.tos-2).Variable_Type_Value=Simulation_Array_Integer)
                	or (
yy.value_stack(yy.tos-2).Variable_Type_Value=Simulation_Array_Boolean) 
                	or (
yy.value_stack(yy.tos-2).Variable_Type_Value=Simulation_Array_String)
                        or (
yy.value_stack(yy.tos-2).Variable_Type_Value=Simulation_Array_Random) 
                        or (
yy.value_stack(yy.tos-2).Variable_Type_Value=Simulation_Time_Unit_Array_Random)
                	or (
yy.value_stack(yy.tos-2).Variable_Type_Value=Simulation_Time_Unit_Array_Double)
                	or (
yy.value_stack(yy.tos-2).Variable_Type_Value=Simulation_Time_Unit_Array_Integer)
                	or (
yy.value_stack(yy.tos-2).Variable_Type_Value=Simulation_Time_Unit_Array_Boolean) 
                	or (
yy.value_stack(yy.tos-2).Variable_Type_Value=Simulation_Time_Unit_Array_String) 
			then

                		New_Array_Expr:= new Array_Variable_Expr;
                                New_Array_Expr.Variable_Type:=
yy.value_stack(yy.tos-2).Variable_Type_Value;
                		New_Array_Expr.Name:=
yy.value_stack(yy.tos-4).String_Value;
				New_Var.Variable:=Variable_Expr_Ptr(New_Array_Expr);
                        else
				New_Var_Expr := new Variable_Expr;
                                New_Var_Expr.Variable_Type:=
yy.value_stack(yy.tos-2).Variable_Type_Value;
				New_Var_Expr.Name:=
yy.value_stack(yy.tos-4).String_Value;
				New_Var.Variable:=Variable_Expr_Ptr(New_Var_Expr);
		end if;



		Add(Variables_Table, New_Var);

               -- Create simulation data
               --
               Variables_Table.Entries(Variables_Table.Nb_Entries-1).Simulation
                       :=new Simulation_Value(
                         Variable_Expr_Ptr(Variables_Table.Entries(Variables_Table.Nb_Entries-1).
                                       Variable).Variable_Type);


		-- If Affectation is not null, we should register
		-- a new Affectation
		--
		if 
yy.value_stack(yy.tos-1).Expr /= null 
			then
				New_Affectation:= new Affectation_Statement;
				New_Affectation.Lvalue:=Expression_Ptr(New_Var_Expr);
                		New_Affectation.Rvalue:=
yy.value_stack(yy.tos-1).Expr;
				New_Affectation.Line_Number:=Scheduler_Lex.Lines;
				
yyval.Stats:=Statement_Ptr(New_Affectation);
                        else 
yyval.Stats:=null;

		end if;
	end;
	

when  39 =>
--#line  295

	declare
		New_Affectation : Affectation_Statement_Ptr;
		
	begin
		New_Affectation:= new Affectation_Statement;
                New_Affectation.Lvalue:=
yy.value_stack(yy.tos-3).Expr;
                New_Affectation.Rvalue:=
yy.value_stack(yy.tos-1).Expr;
		New_Affectation.Line_Number:=Scheduler_Lex.Lines;

		
yyval.Stats:=Statement_Ptr(New_Affectation);

	end;
	

when  40 =>
--#line  311


		New_If:= new If_Statement;
                New_If.Bool_Expression:=
yy.value_stack(yy.tos-2).Expr;
		New_If.Line_Number:=Scheduler_Lex.Lines;

                New_If.Then_Statement:=Then_Inner_Statement_Ptr(Then_Inner_Statement_Index);
                Then_Inner_Statement_Index:=Then_Inner_Statement_Index-1;

		
yyval.Stats:=Statement_Ptr(New_If);

                New_Stat:=Copy(New_If.Then_Statement);
                New_Stat.Next_Statement:=null;
                while  New_If.Then_Statement.Next_Statement/=null loop
                	Old_Stat:=Copy(New_If.Then_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                	New_If.Then_Statement:=New_If.Then_Statement.Next_Statement;
                end loop;
                New_If.Then_Statement:=New_Stat;


        

when  41 =>
--#line  335


                New_If.Else_Statement:=Else_Inner_Statement_Ptr(Else_Inner_Statement_Index);
                Else_Inner_Statement_Index:=Else_Inner_Statement_Index-1;
		
yyval.Stats:=Statement_Ptr(New_If);

                New_Stat:=Copy(New_If.Else_Statement);
                New_Stat.Next_Statement:=null;
                while  New_If.Else_Statement.Next_Statement/=null loop
                	Old_Stat:=Copy(New_If.Else_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                	New_If.Else_Statement:=New_If.Else_Statement.Next_Statement;
                end loop;
                New_If.Else_Statement:=New_Stat;
        

when  42 =>
--#line  354

	declare
		New_Return : Return_Statement_Ptr;
		
	begin
		New_Return:= new Return_Statement;
                New_Return.Return_Value:=
yy.value_stack(yy.tos).Expr;
		New_Return.Line_Number:=Scheduler_Lex.Lines;

		
yyval.Stats:=Statement_Ptr(New_Return);

	end;
	

when  43 =>
--#line  370

                New_for:= new for_Statement;
                New_for.for_Type:=
yy.value_stack(yy.tos-5).Table_Type_Value;
                New_for.for_Index:=Variable_Expr_Ptr(
yy.value_stack(yy.tos-7).Expr);
                New_for.Line_Number:=Scheduler_Lex.Lines;
                 
                New_for.Included_Statement:=Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index);
                Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index-1;

                New_Stat:=Copy(New_for.Included_Statement);
                New_Stat.Next_Statement:=null;
                while  New_for.Included_Statement.Next_Statement/=null loop
                        Old_Stat:=Copy(New_for.Included_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                        New_for.Included_Statement:=New_for.Included_Statement.Next_Statement;
                end loop;
                New_for.Included_Statement:=New_Stat;

                
yyval.Stats:=Statement_Ptr(New_for);
        

when  44 =>
--#line  394

                New_While:= new While_Statement;
                New_While.Condition:=
yy.value_stack(yy.tos-5).Expr;
                New_While.Line_Number:=Scheduler_Lex.Lines;
                 
                New_While.Included_Statement:=Loop_Inner_Statement_Ptr(Loop_Inner_Statement_Index);
                Loop_Inner_Statement_Index:=Loop_Inner_Statement_Index-1;

                New_Stat:=Copy(New_While.Included_Statement);
                New_Stat.Next_Statement:=null;
                while  New_While.Included_Statement.Next_Statement/=null loop
                	Old_Stat:=Copy(New_While.Included_Statement.Next_Statement);
                	Old_Stat.Next_Statement:=New_Stat;
                	New_Stat:=Old_Stat;
                	New_While.Included_Statement:=New_While.Included_Statement.Next_Statement;
                end loop;
                New_While.Included_Statement:=New_Stat;

                
yyval.Stats:=Statement_Ptr(New_While);

        

when  45 =>
--#line  418

	declare 
                New_Random_Statement : Random_Initialize_Statement_Ptr;
	begin
                New_Random_Statement:=new Random_Initialize_Statement;
                New_Random_Statement.Lvalue:=
yy.value_stack(yy.tos-4).String_Value;
                New_Random_Statement.Law:=Exponential_Law;
                New_Random_Statement.Parameter1:=
yy.value_stack(yy.tos-2).Expr;   
                
yyval.Stats:=Statement_Ptr(New_Random_Statement);
	end;
        

when  46 =>
--#line  432

	declare 
                New_Random_Statement : Random_Initialize_Statement_Ptr;
	begin
                New_Random_Statement:=new Random_Initialize_Statement;
                New_Random_Statement.Lvalue:=
yy.value_stack(yy.tos-6).String_Value;
                New_Random_Statement.Law:=Uniform_Law;
                New_Random_Statement.Parameter1:=
yy.value_stack(yy.tos-4).Expr;
                New_Random_Statement.Parameter2:=
yy.value_stack(yy.tos-2).Expr; 
                
yyval.Stats:=Statement_Ptr(New_Random_Statement);
	end;
	

when  47 =>
--#line  447

        declare
                New_Set : Set_Statement_Ptr;
        begin
                -- Is the Variable already exists ?
                --
                for I in 0..Sets_Table.Nb_Entries-1 loop
                        if (Sets_Table.Entries(i).Set_Id=
yy.value_stack(yy.tos-2).String_Value)
                                then
                                        Raise_Exception(Expressions.Syntax_Error'Identity, To_String(
                         
yy.value_stack(yy.tos-2).String_Value & Lb_Comma & Lb_Identifier_Already_Declared(Current_Language) ));
                        end if;
                end loop;

                New_Set := new Set_Statement;
                New_Set.Set_Value:= 
yy.value_stack(yy.tos-1).Expr;
                New_Set.Set_Id := 
yy.value_stack(yy.tos-2).String_Value;
                New_Set.Line_Number := Scheduler_Lex.Lines;

		Add(Sets_Table, New_Set);
	
                
yyval.Stats:= Statement_Ptr(New_Set);
        end;
        

when  48 =>
--#line  474

                
yyval.Integer_Value:=
yy.value_stack(yy.tos).Integer_Value;
	

when  49 =>
--#line  478

                
yyval.Integer_Value:=0;
        

when  50 =>
--#line  484

		null;
	

when  51 =>
--#line  488

        declare
                New_Nop : Nop_Statement_Ptr;

        begin
                New_Nop:= new Nop_Statement;
                New_Nop.Line_Number:=Scheduler_Lex.Lines;

                Else_Inner_Statement_Index:=Else_Inner_Statement_Index+1;
                Else_Inner_Statement_Ptr(Else_Inner_Statement_Index):=Statement_Ptr(New_Nop);

        end;

	

when  52 =>
--#line  505

		
yyval.Expr:=
yy.value_stack(yy.tos-1).Expr;
	

when  53 =>
--#line  509

		
yyval.Expr:=null;
	

when  54 =>
--#line  517

		
yyval.Variable_Type_Value:=Simulation_Double;
	

when  55 =>
--#line  521

                if 
yy.value_stack(yy.tos-3).Table_Type_Value=Time_Unit_Table_Type
                   then 
                      
yyval.Variable_Type_Value:=Simulation_Time_Unit_Array_Double;
                   else
                      
yyval.Variable_Type_Value:=Simulation_Array_Double;
                end if;
	

when  56 =>
--#line  530

                
yyval.Variable_Type_Value:=Simulation_Integer;
	

when  57 =>
--#line  534

                if 
yy.value_stack(yy.tos-3).Table_Type_Value=Time_Unit_Table_Type
                   then 
                      
yyval.Variable_Type_Value:=Simulation_Time_Unit_Array_Integer;
                   else
                      
yyval.Variable_Type_Value:=Simulation_Array_Integer;
                end if; 
	

when  58 =>
--#line  543

                
yyval.Variable_Type_Value:=Simulation_Random;
	

when  59 =>
--#line  547

                if 
yy.value_stack(yy.tos-3).Table_Type_Value=Time_Unit_Table_Type
                   then 
                      
yyval.Variable_Type_Value:=Simulation_Time_Unit_Array_Random;
                   else
                      
yyval.Variable_Type_Value:=Simulation_Array_Random;
                end if; 
	

when  60 =>
--#line  556

                
yyval.Variable_Type_Value:=Simulation_Boolean;
	

when  61 =>
--#line  560

                if 
yy.value_stack(yy.tos-3).Table_Type_Value=Time_Unit_Table_Type
                   then 
                      
yyval.Variable_Type_Value:=Simulation_Time_Unit_Array_Boolean;
                   else
                      
yyval.Variable_Type_Value:=Simulation_Array_Boolean;
                end if; 
	

when  62 =>
--#line  569

                if 
yy.value_stack(yy.tos-3).Table_Type_Value=Time_Unit_Table_Type
                   then 
                      
yyval.Variable_Type_Value:=Simulation_Time_Unit_Array_String;
                   else
                      
yyval.Variable_Type_Value:=Simulation_Array_String;
                end if; 
	

when  63 =>
--#line  578

                
yyval.Variable_Type_Value:=Simulation_String;
	

when  64 =>
--#line  586

                
yyval.Table_Type_Value:=Task_Table_Type;
	

when  65 =>
--#line  590

                
yyval.Table_Type_Value:=Buffer_Table_Type;
	

when  66 =>
--#line  594

                
yyval.Table_Type_Value:=Message_Table_Type;
	

when  67 =>
--#line  598

                
yyval.Table_Type_Value:=Time_Unit_Table_Type;
	

when  68 =>
--#line  602

                
yyval.Table_Type_Value:=Processor_Table_Type;
	

when  69 =>
--#line  606

                
yyval.Table_Type_Value:=Resource_Table_Type;
	

when  70 =>
--#line  614

        	
yyval.Expr:=
yy.value_stack(yy.tos).Expr;
	

when  71 =>
--#line  618

		
yyval.Expr:=null;
	

when  72 =>
--#line  625

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
		New_Expr.Ope:=Logic_And;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  73 =>
--#line  638

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
                New_Expr.Ope:=Modulo;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;            
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  74 =>
--#line  651

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
                New_Expr.Ope:=Inf;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;            
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  75 =>
--#line  664

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
		New_Expr.Ope:=Sup;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  76 =>
--#line  677

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
		New_Expr.Ope:=Equal_Less;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  77 =>
--#line  690

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
		New_Expr.Ope:=Equal_Greater;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  78 =>
--#line  703

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
		New_Expr.Ope:=Not_Equal;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  79 =>
--#line  716

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
		New_Expr.Ope:=Equal;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  80 =>
--#line  729

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
		New_Expr.Ope:=Logic_Or;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  81 =>
--#line  742

	declare
                New_Expr : Unary_Expr_Ptr; 
	begin
                New_Expr:=new Unary_Expr;
		New_Expr.Ope:=Logic_Not;
                New_Expr.Value:=
yy.value_stack(yy.tos-1).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  82 =>
--#line  754

        	
yyval.Expr:=
yy.value_stack(yy.tos-1).Expr;
	

when  83 =>
--#line  759

	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
		New_Expr:=new Constant_Expr;
		New_Expr.Value:= new Simulation_Value'(Simulation_Boolean, true);
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  84 =>
--#line  770

	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
		New_Expr:=new Constant_Expr;
		New_Expr.Value:= new Simulation_Value'(Simulation_Boolean, false);
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  85 =>
--#line  781

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
		New_Expr.Ope:=Plus;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  86 =>
--#line  794

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
		New_Expr.Ope:=Minus;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  87 =>
--#line  807

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
		New_Expr.Ope:=Mult;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  88 =>
--#line  820

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
                New_Expr.Ope:=Div;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  89 =>
--#line  833

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
                New_Expr.Ope:=Lcm;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-5).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos-3).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  90 =>
--#line  847

        declare
                New_Expr : Unary_Expr_Ptr;
        begin
                New_Expr:=new Unary_Expr;
                New_Expr.Value:=
yy.value_stack(yy.tos-1).Expr;
                New_Expr.ope:=Abs_Operator;
                
yyval.Expr:=Expression_Ptr(New_Expr);
        end;
        

when  91 =>
--#line  860

        declare
                New_Expr : Binary_Expr_Ptr;
        begin
                New_Expr:=new Binary_Expr;
                New_Expr.Ope:=Concat;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-5).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos-3).Expr;
                
yyval.Expr:=Expression_Ptr(New_Expr);
        end;
	

when  92 =>
--#line  873

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
                New_Expr.Ope:=Max_Operator;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-3).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos-1).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  93 =>
--#line  886

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
                New_Expr.Ope:=Min_Operator;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-3).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos-1).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  94 =>
--#line  899

	declare
                New_Expr : Binary_Expr_Ptr; 
	begin
                New_Expr:=new Binary_Expr;
		New_Expr.Ope:=Exp;
                New_Expr.Lvalue:=
yy.value_stack(yy.tos-2).Expr;
                New_Expr.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  95 =>
--#line  912

	declare
                New_Expr1 : Binary_Expr_Ptr; 
		New_Expr2 : Constant_Expr_Ptr;	

	begin
                New_Expr2:=new Constant_Expr;
                if Get_Type(
yy.value_stack(yy.tos).Expr.all) = Simulation_Double then
                        New_Expr2.value:=new Simulation_Value'(Simulation_Double,
                                        -1.0);
                else
                        New_Expr2.value:=new Simulation_Value'(Simulation_Integer,
                                        -1);
                end if;
                New_Expr1:=new Binary_Expr;
		New_Expr1.Ope:=Mult;
		New_Expr1.Lvalue:=Expression_Ptr(New_Expr2);
                New_Expr1.Rvalue:=
yy.value_stack(yy.tos).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr1);
	end;
	

when  96 =>
--#line  935

	declare 
                New_Expr : Unary_Expr_Ptr;
	begin
                New_Expr := new Unary_Expr;
                New_Expr.value:=
yy.value_stack(yy.tos-1).Expr;
                New_Expr.Ope:=Max_To_Index;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  97 =>
--#line  947

	declare 
                New_Expr : Unary_Expr_Ptr;
	begin
                New_Expr := new Unary_Expr;
                New_Expr.value:=
yy.value_stack(yy.tos-1).Expr;
                New_Expr.Ope:=Min_To_Index;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  98 =>
--#line  960

	declare 
                New_Expr : Unary_Expr_Ptr;
	begin
                New_Expr := new Unary_Expr;
                New_Expr.value:=
yy.value_stack(yy.tos-1).Expr;
                New_Expr.Ope:=Get_Task_Index;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  99 =>
--#line  972

	declare 
                New_Expr : Unary_Expr_Ptr;
	begin
                New_Expr := new Unary_Expr;
                New_Expr.value:=
yy.value_stack(yy.tos-1).Expr;
                New_Expr.Ope:=Get_Resource_Index;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  100 =>
--#line  984

	declare 
                New_Expr : Unary_Expr_Ptr;
	begin
                New_Expr := new Unary_Expr;
                New_Expr.value:=
yy.value_stack(yy.tos-1).Expr;
                New_Expr.Ope:=Get_Buffer_Index;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  101 =>
--#line  996

	declare 
                New_Expr : Unary_Expr_Ptr;
	begin
                New_Expr := new Unary_Expr;
                New_Expr.value:=
yy.value_stack(yy.tos-1).Expr;
                New_Expr.Ope:=Get_Message_Index;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  102 =>
--#line  1008

	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
                New_Expr:=new Constant_Expr;
                New_Expr.value:=new Simulation_Value'(Simulation_Double,Double'Last);
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  103 =>
--#line  1018

	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
                New_Expr:=new Constant_Expr;
                New_Expr.value:=new Simulation_Value'(Simulation_Double,Double'First);
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  104 =>
--#line  1028

	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
                New_Expr:=new Constant_Expr;
                New_Expr.value:=new Simulation_Value'(Simulation_Integer,Integer'Last);
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  105 =>
--#line  1038

	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
                New_Expr:=new Constant_Expr;
                New_Expr.value:=new Simulation_Value'(Simulation_Integer,Integer'First);
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  106 =>
--#line  1049

	declare
		New_Expr : Constant_Expr_Ptr;	
		Tmp : Unbounded_string;
	begin
                New_Expr:=new Constant_Expr;
		Tmp:=To_Unbounded_String(Scheduler_Dfa.Yytext);
		Tmp:=To_Unbounded_String(Slice(Tmp, 2, Length(Tmp)-1));
                New_Expr.value:=new Simulation_Value'(Simulation_String,Tmp);
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  107 =>
--#line  1063

	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
                New_Expr:=new Constant_Expr;
                New_Expr.value:=new Simulation_Value'(Simulation_Double,
yy.value_stack(yy.tos).Double_Value);
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  108 =>
--#line  1074

	declare
		New_Expr : Constant_Expr_Ptr;	
	begin
                New_Expr:=new Constant_Expr;
                New_Expr.value:=new Simulation_Value'(Simulation_Integer,
yy.value_stack(yy.tos).Integer_Value);
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  109 =>
--#line  1085

	declare
		New_Expr : Array_Variable_Expr_Ptr;	
	begin
		-- Find the type of the $1 Variable
		--
		
yyval.Expr:=Expression_Ptr(Variables_Table.Entries(
                        Find_Variable(Variables_Table, 
yy.value_stack(yy.tos-3).String_Value)).Variable );
		
		New_Expr := new 
			Array_Variable_Expr;
		New_Expr.Variable_Type:=Variable_Expr_Ptr(
yyval.Expr).Variable_Type;
                New_Expr.Name:=
yy.value_stack(yy.tos-3).String_Value;
                New_Expr.Array_index:=
yy.value_stack(yy.tos-1).Expr;
		
yyval.Expr:=Expression_Ptr(New_Expr);
	end;
	

when  110 =>
--#line  1104

		-- Find the type of the Variable before
		-- instanciation
		--
		
yyval.Expr:=Expression_Ptr(Variables_Table.Entries(
		Find_Variable(Variables_Table, 
yy.value_stack(yy.tos).String_Value)).Variable );
	

when  111 =>
--#line  1117

		
yyval.String_Value := To_Unbounded_String(Scheduler_Dfa.Yytext); 
        

when  112 =>
--#line  1123
 
	declare 
		Ok : boolean;

	begin
		To_Double(To_Unbounded_String(Scheduler_Dfa.Yytext),
yyval.Double_Value,Ok);
         	if not Ok
               		then
				Raise_Exception(Expressions.Syntax_Error'Identity, To_String(
			 		To_Unbounded_String(Scheduler_Dfa.Yytext) & Lb_Comma & Lb_Double_Conversion_Error(Current_Language) ));
         	end if;
         end;
	

when  113 =>
--#line  1139
 
	declare 
		Ok : boolean;

	begin
		To_Integer(To_Unbounded_String(Scheduler_Dfa.Yytext),
yyval.Integer_Value,Ok);
		if not Ok
			then
				Raise_Exception(Expressions.Syntax_Error'Identity, To_String(
					To_Unbounded_String(Scheduler_Dfa.Yytext) & Lb_Comma & Lb_Integer_Conversion_Error(Current_Language) ));
         	end if;
	end;
	

                    when others => null;
                end case;


            -- Pop RHS states and goto next state
            yy.tos      := yy.tos - rule_length(yy.rule_id) + 1;
            if yy.tos > yy.stack_size then
                text_io.put_line(" Stack size exceeded on state_stack");
                raise yy_Tokens.syntax_error;
            end if;
            yy.state_stack(yy.tos) := goto_state(yy.state_stack(yy.tos-1) ,
                                 get_lhs_rule(yy.rule_id));

              yy.value_stack(yy.tos) := yyval;

            if yy.debug then
                reduce_debug(yy.rule_id,
                    goto_state(yy.state_stack(yy.tos - 1),
                               get_lhs_rule(yy.rule_id)));
            end if;

        end if;


    end loop;


end yyparse;



end Parser;

