------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

package body Queueing_System.Theoritical.Mms is

   procedure Initialize (
         A_Queueing_System : in out Mms_Queueing_System_Theoritical ) is 
   begin
      Initialize(Generic_Queueing_System_Theoritical(A_Queueing_System));

      A_Queueing_System.Queueing_System_Type := Qs_Mms;
   end Initialize;

   procedure Qs_Average_Waiting_Time (
         A_Queueing_System : in     Mms_Queueing_System_Theoritical; 
         Result            : in out Double                           ) is 

      Arrival_Rate : Double := A_Queueing_System.Arrival_Rate.Entries (0).Data;  
      Service_Rate : Double := A_Queueing_System.Service_Rate.Entries (0).Data;  
   begin
      Result := 1.0 / (Service_Rate - Arrival_Rate) - (1.0 / Service_Rate);
   end Qs_Average_Waiting_Time;

   procedure Qs_Average_Number_Customer (
         A_Queueing_System : in     Mms_Queueing_System_Theoritical; 
         Result            : in out Double                           ) is 

      Rau : Double := A_Queueing_System.Arrival_Rate.Entries (0).Data / A_Queueing_System.Service_Rate.Entries
        (0).Data;  
   begin
      Result := Rau / (1.0 - Rau);
   end Qs_Average_Number_Customer;

   procedure Qs_Maximum_Waiting_Time (
         A_Queueing_System : in     Mms_Queueing_System_Theoritical; 
         Result            : in out Double                           ) is 
   begin
      raise Not_Implemented;
   end Qs_Maximum_Waiting_Time;

   procedure Qs_Maximum_Number_Customer (
         A_Queueing_System : in     Mms_Queueing_System_Theoritical; 
         Result            : in out Double                           ) is 
   begin
      raise Not_Implemented;
   end Qs_Maximum_Number_Customer;

   function Get_Probability_Of_State (
         A_Queueing_System : in     Mms_Queueing_System_Theoritical; 
         N                 : in     Natural                          ) 
     return Double is 
      Utilisation : Double;  
      Result      : Double;  
   begin
      Utilisation := A_Queueing_System.Arrival_Rate.Entries(0).Data /
         A_Queueing_System.Service_Rate.Entries(0).Data;
      Result := Pow(Utilisation,Double(N));
      Result := Result * (1.0 - Utilisation);
      return (Result);
   end;

   function Get_Qs_Nb_Tasks (
         A_Queueing_System : in     Mms_Queueing_System_Theoritical ) 
     return Double is 
   begin
      return  A_Queueing_System.Nb_Tasks;
   end;

   procedure Set_Qs_Nb_Tasks (
         A_Queueing_System : in out Mms_Queueing_System_Theoritical; 
         Value             :        Double                           ) is 
   begin
      A_Queueing_System.Nb_Tasks := Value;
   end;

   function Get_Probability_Of_Queueing (
         A_Queueing_System : in     Mms_Queueing_System_Theoritical ) 
     return Double is 
      Utilisation  : Double;  
      Adivs        : Double;  
      Upper_Result : Double := 0.0;  
      Lower_Result : Double := 0.0;  
      Lambda       : Double := 0.0;  

      function Factorial (
            Value : in     Double ) 
        return Double is 
         N      : Double := Value;  
         Result : Double := Value;  
      begin
         while (N > 1.0) loop
            --            Put(" " & N'Img);
            N := N - 1.0;
            Result := Result * N;
         end loop;
         return Result;
      end Factorial;

   begin

      Utilisation := A_Queueing_System.Arrival_Rate.Entries(0).
         Data /
         A_Queueing_System.Service_Rate.Entries(0).Data;

      Adivs :=  Lambda /
         A_Queueing_System.Service_Rate.Entries(0).Data /
         A_Queueing_System.Nb_Tasks;

      Upper_Result := (1.0 / Factorial(A_Queueing_System.Nb_Tasks)) *
         (Pow(Adivs,A_Queueing_System.Nb_Tasks)) *
         (1.0 / (1.0 - Utilisation));

      Lower_Result := Lower_Result + 1.0;
      for N in 1 .. Natural(A_Queueing_System.Nb_Tasks)-1 loop
         Lower_Result := Lower_Result +
            (1.0 / Factorial(Double(N)) * Pow(Adivs,Double(N)));
      end loop;

      Lower_Result := Lower_Result +
         (1.0 / Factorial(A_Queueing_System.Nb_Tasks)) *
         (Pow(Adivs,A_Queueing_System.Nb_Tasks)) *
         (1.0 / (1.0 - Utilisation));

      return (Upper_Result / Lower_Result);
   end;

end Queueing_System.Theoritical.Mms;



