------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Text_Io;
use Text_Io;

package body Queueing_System.Theoritical is

   procedure Theoritical_Results (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical'Class; 
         Message           : in     Unbounded_String                           ) is 

      Awt,  
      Anc,  
      Mwt,  
      Mnc : Double := 0.0;  
   begin
      -- criteria computation
      Qs_Average_Waiting_Time(A_Queueing_System,Awt);
      Qs_Average_Number_Customer (A_Queueing_System,Anc);
      Qs_Maximum_Waiting_Time (A_Queueing_System,Mwt);
      Qs_Maximum_Number_Customer (A_Queueing_System,Mnc);

      -- criteria display
      Put_Line("Theoritical Results");
      New_Line;
      Put_Line("AW" & To_String(Message) & " " &
         A_Queueing_System.Utilisation'Img & " " & Awt'Img);
      Put_Line("AO" & To_String(Message) & " " &
         A_Queueing_System.Utilisation'Img & " " & Anc'Img);
      Put_Line("MW" & To_String(Message) & " " &
         A_Queueing_System.Utilisation'Img & " " & Mwt'Img);
      Put_Line("MO" & To_String(Message) & " " &
         A_Queueing_System.Utilisation'Img & " " & Mnc'Img);
      New_Line;
   end Theoritical_Results ;

   procedure Initialize (
         A_Queueing_System : in out Generic_Queueing_System_Theoritical ) is 

   begin
      Initialize(Generic_Queueing_System(A_Queueing_System));
   end Initialize;

   procedure Reset (
         A_Queueing_System : in out Generic_Queueing_System_Theoritical ) is 

   begin
      Initialize (Generic_Queueing_System_Theoritical(A_Queueing_System));
   end Reset;

   function Get_Probability_Density (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical'Class;     
         N                 : in     Natural                                   := 0 ) 
     return Double is 
      State  : Natural;  
      Result : Double  := 0.0;  
   begin
      State := N;
      if State > A_Queueing_System.Max_State then
         return (-1.0);
      end if;
      if State=0 then
         State := A_Queueing_System.Max_State;
      end if;

      for I in 0 .. State loop
         Result := Result + Get_Probability_Of_State(A_Queueing_System,I);
      end loop;

      return Result;
   end;

   function Get_Probability_Of_Full_Buffer (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical ) 
     return Double is 
   begin
      raise Not_Implemented;
      return 0.0;
   end;

   function Get_Probability_Of_Task_Block (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical ) 
     return Double is 
   begin
      raise Not_Implemented;
      return 0.0;
   end;

   function Get_Probability_Of_Queueing (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical ) 
     return Double is 
   begin
      raise Not_Implemented;
      return 0.0;
   end;

end Queueing_System.Theoritical;
