------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automaticly check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Numerics.Aux;
use Ada.Numerics.Aux;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

package Queueing_System.Theoritical is

   type Generic_Queueing_System_Theoritical is abstract new Generic_Queueing_System with null record; 
   type Queueing_System_Theoritical_Ptr is access all Generic_Queueing_System_Theoritical'Class; 


   procedure Initialize (
         A_Queueing_System : in out Generic_Queueing_System_Theoritical ); 

   -- reset caracteristics of the qs (has to be done each time a qs is used)
   --
   procedure Reset (
         A_Queueing_System : in out Generic_Queueing_System_Theoritical ); 

   -- return the theoric average waiting time of one customer in the queue
   -- based on theoric service time
   --
   procedure Qs_Average_Waiting_Time (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical; 
         Result            : in out Double                               ) is 
   abstract;

   -- return the theoric average number of customer in the queue
   -- based on theoric service time
   --
   procedure Qs_Average_Number_Customer (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical; 
         Result            : in out Double                               ) is 
   abstract;

   -- return the theoric maximum waiting time of one customer in the queue
   --
   procedure Qs_Maximum_Waiting_Time (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical; 
         Result            : in out Double                               ) is 
   abstract;

   -- return the theoric maximum number of customer in the queue
   --
   procedure Qs_Maximum_Number_Customer (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical; 
         Result            : in out Double                               ) is 
   abstract;

   -- return theoritical results on qs (waiting time, number of customer)
   --
   procedure Theoritical_Results (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical'Class; 
         Message           : in     Unbounded_String                           ); 

   -- return the probability to be in a certain state
   --
   function Get_Probability_Of_State (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical; 
         N                 : in     Natural                              ) 
     return Double is 
   abstract;

   -- return the probability of buffer will be full
   --
   function Get_Probability_Of_Full_Buffer (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical ) 
     return Double; 

   -- return the probability of all tasks are busy
   -- Eq. 2.153, Robertazzi: Queueing theory
   --
   function Get_Probability_Of_Task_Block (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical ) 
     return Double; 

   --P[queueing], Eq. 2.148, Robertazzi
   --
   function Get_Probability_Of_Queueing (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical ) 
     return Double; 

   --Get probability density of states from 0->N
   --
   function Get_Probability_Density (
         A_Queueing_System : in     Generic_Queueing_System_Theoritical'Class;     
         N                 : in     Natural                                   := 0 ) 
     return Double; 

end Queueing_System.Theoritical;




