------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automatically check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Text_Io;
use Text_Io;

package body Resources is


   procedure Put_Name (
         A_Resource : in     Resource_Ptr ) is 
   begin
      Put(To_String(A_Resource.Name));
   end Put_Name;

   procedure Put (
         A_Resource : in     Resource_Ptr ) is 
   begin
      Put(A_Resource.all);
   end Put;


   procedure Put (
         My_Resource : in     Generic_Resource ) is 

   begin

      Put("Name: "& To_String(My_Resource.Name));
      Put("; Cpu : "& To_String(My_Resource.Cpu_Name));
      Put("; Address space : "& To_String(My_Resource.Address_Space_Name));
      Put("; Protocol : "& My_Resource.Protocol'Img);
      Put_Line("; State : "& To_String(To_Unbounded_String(Natural'Image(
                  My_Resource.State))));

      Put_Line("Affected Tasks :");
      Put_Line("Task_Name   Begin   End");
      Put(My_Resource.Task_Tab);


   end Put;



   function Copy (
         My_Resource : in     Resource_Ptr ) 
     return Resource_Ptr is 

      New_Np_Resource  : Np_Resource_Ptr;  
      New_Pcp_Resource : Pcp_Resource_Ptr;  
      New_Pip_Resource : Pip_Resource_Ptr;  

      Ptr : Resource_Ptr;  

   begin
      if (My_Resource.Protocol = No_Protocol)
            then
         New_Np_Resource:=new Np_Resource;
         New_Np_Resource.all:=Np_Resource_Ptr(My_Resource).all;
         Ptr:=Resource_Ptr(New_Np_Resource);
      else
         if (My_Resource.Protocol = Priority_Ceiling_Protocol)
               then
            New_Pcp_Resource:=new Pcp_Resource;
            New_Pcp_Resource.all:=Pcp_Resource_Ptr(My_Resource).all;
            Ptr:=Resource_Ptr(New_Pcp_Resource);
         else
            New_Pip_Resource:=new Pip_Resource;
            New_Pip_Resource.all:=Pip_Resource_Ptr(My_Resource).all;
            Ptr:=Resource_Ptr(New_Pip_Resource);
         end if;
      end if;

      return Ptr;
   end Copy;


   procedure Put (
         A : in     Affected_Task_List ) is 
   begin
      Put(A.Task_Begin'Img & "/" & A.Task_End'Img);
   end Put;

   procedure Initialize (
         A :    out Affected_Task_List ) is 
   begin
      A.Task_Begin:=0;
      A.Task_End:=0;
   end Initialize;


   procedure Initialize (
         A_Resource : in out Generic_Resource ) is 
   begin
      A_Resource.Object_Type:=Resource_Object_Type;
   end Initialize;


   procedure Initialize (
         A_Resource : in out Pip_Resource ) is 
   begin
      Initialize(Generic_Resource(A_Resource));
      A_Resource.Protocol:=Priority_Inheritance_Protocol;
   end Initialize;



   procedure Initialize (
         A_Resource : in out Np_Resource ) is 
   begin
      Initialize(Generic_Resource(A_Resource));
      A_Resource.Protocol:=No_Protocol;
   end Initialize;



   procedure Initialize (
         A_Resource : in out Pcp_Resource ) is 
   begin
      Initialize(Generic_Resource(A_Resource));
      A_Resource.Protocol:=Priority_Ceiling_Protocol;
   end Initialize;



end Resources;



