------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automatically check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;
with Scheduler.Dynamic_Priority;


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;




package Scheduler.Dynamic_Priority.Llf is


   type Llf_Scheduler is new Dynamic_Priority_Scheduler with private; 
   type Llf_Scheduler_Ptr is access all Llf_Scheduler'Class; 


   procedure Initialize (
         A_Scheduler : in out Llf_Scheduler ); 


   function Copy (
         A_Scheduler : in     Llf_Scheduler ) 
     return Scheduler_Ptr; 




   procedure Do_Election (
         My_Scheduler      : in out Llf_Scheduler;           
         Si                : in out Scheduling_Information;  
         Result            : in out Scheduling_Sequence_Ptr; 
         Msg               : in out Unbounded_String;        
         Current_Time      : in     Natural;                 
         Processor_Name    : in     Unbounded_String;        
         My_Dependencies   : in     Tasks_Dependencies_Ptr;  
         With_Offsets      : in      Boolean;                 
         With_Precedencies :  in     Boolean;                 
         With_Resources    :   in    Boolean;                 
        Event_To_Generate          : in Time_Unit_Event_Type_Boolean_Table;
         Elected           : in out Tasks_Range;             
         No_Task           : in out Boolean                  ); 


   procedure Utilization_Factor_Feasibility_Test (
         My_Scheduler   : in     Llf_Scheduler;    
         My_Tasks       : in     Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Result         : in out Unbounded_String  ); 




private

   type Llf_Scheduler is new Dynamic_Priority_Scheduler with null record; 

end Scheduler.Dynamic_Priority.Llf ;


