------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with Ada.Text_Io;
use Ada.Text_Io;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


with Config;

with Natural_Util;
use Natural_Util;

with Tasks;
use Tasks;

with Indexed_Tables;


package body Time_Units is



   procedure Put (
         E : in     Time_Unit_Event_Ptr ) is 
   begin
      if (E/=null) then
         Put(E.Event_Type'Img & '/');
         case E.Event_Type is
            when Start_Of_Task_Capacity =>
               Put(To_String(E.Start_Task.Name));
            when End_Of_Task_Capacity =>
               Put(To_String(E.End_Task.Name));
            when Write_To_Buffer =>
               Put(To_String(E.Write_Buffer.Name));
               Put("/");
               Put(To_String(E.Write_Task.Name));
               Put("/" & E.Write_Size'Img);
            when Read_From_Buffer =>
               Put(To_String(E.Read_Buffer.Name));
               Put("/");
               Put(To_String(E.Read_Task.Name));
               Put("/" & E.Read_Size'Img);
            when Running_Task =>
               Put(To_String(E.Running_Task.Name));
            when Task_Activation =>
               Put(To_String(E.Activation_Task.Name));
            when Allocate_Resource =>
               Put(To_String(E.Allocate_Resource.Name));
               Put("/");
               Put(To_String(E.Allocate_Task.Name));
            when Release_Resource =>
               Put(To_String(E.Release_Resource.Name));
               Put("/");
               Put(To_String(E.Release_Task.Name));
            when Wait_For_Resource =>
               Put(To_String(E.Wait_Resource.Name));
               Put("/");
               Put(To_String(E.Wait_Task.Name));
            when Send_Message =>
               Put(To_String(E.Send_Message.Name));
            when Receive_Message =>
               Put(To_String(E.Receive_Message.Name));
         end case;
      end if;
   end Put;

   procedure Put_Line (
         E : in     Time_Unit_Event_Ptr ) is 
   begin
      if (E /= null) then
         Put(E);
         New_Line;
      end if;
   end Put_Line;


   procedure Initialize (
         E : in out Time_Unit_Event_Ptr ) is 
   begin
      E := null;
   end Initialize;


   procedure Initialize (
         E : in out Time_Unit_Event_Type_Boolean_Table; val : in boolean) is 
   begin
      for I in   e'range loop
         e(i):=val;
      end loop;
   end Initialize;



end Time_Units;


