------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Config;
use Config;
with Translate;
use Translate;

with Gtk.Check_Button;
use Gtk.Check_Button;


with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Gtk.Button;
use Gtk.Button;
with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;

with Project;
use Project;
with Glib;
use Glib;

with Text_Io;
use Text_Io;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;

with Generic_Select_File_Pkg;



package body Do_Scheduling_Simulation_Pkg.Callbacks is




   procedure Read_Sc_File (
         File_Name : in     Unbounded_String ) is 

   begin
      Set_Text(Do_Scheduling_Simulation.
         Event_Table_File_Name,
         To_String(File_Name) );
   end Read_Sc_File;



   procedure Dummy is 
   begin
      null;
   end Dummy;

   package Read_Sc_File_Package is new Generic_Select_File_Pkg
      (Read_Sc_File, Dummy);
   use Read_Sc_File_Package;



   procedure On_Read_From_File_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

   begin

      Gtk_New (Read_Sc_File_Package.Generic_Select_File,  Lb_Read(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language) );
      Show_All (Read_Sc_File_Package.Generic_Select_File);


   end On_Read_From_File_Pressed;



   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      A_Processor : Processor_Ptr;  
      Cpu_Index   : Gint;  


   begin


      -- Get customization provided by the user
      --
      Context_Switch_From_Simulation :=
         Get_Active(Do_Scheduling_Simulation.Context_Switch);
      Preemption_From_Simulation:=
         Get_Active(Do_Scheduling_Simulation.Number_Preemption);
      Run_Event_Analyzer_From_Simulation:=
         Get_Active(Do_Scheduling_Simulation.Event_Analyzer);
      Schedule_And_Display_Event_Table:=
         Get_Active(Do_Scheduling_Simulation.
         Schedule_And_Display_Event_Table);
      Schedule_And_Export_Event_Table:=
         Get_Active(Do_Scheduling_Simulation.
         Schedule_And_Export_Event_Table);
      Event_Table_File_Name:=To_Unbounded_String(Get_Text(
            Do_Scheduling_Simulation.Event_Table_File_Name));
      Schedule_All_Processeurs:=
         Get_Active(Do_Scheduling_Simulation.Schedule_All_Processeurs);
      Xml_Event_Table_File_Name:= To_Unbounded_String(Get_Text(
            Do_Scheduling_Simulation.
            Event_Table_File_Name));


      for I in Data_From_Simulation_Table'range loop
         Blocking_Time_From_Simulation(I):=
            Get_Active(Do_Scheduling_Simulation.Blocking_Time(I));
         Response_Time_From_Simulation(I):=
            Get_Active(Do_Scheduling_Simulation.Response_Time(I));
      end loop;


      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (
               Do_Scheduling_Simulation.Clist_Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors, A_Processor, Processors_Range(
            Cpu_Index));
      Last_Selected_Processor :=A_Processor.Name;



      Do_Scheduling_Simulation.Call.all;


      Hide(Do_Scheduling_Simulation);

   end On_Ok_Pressed;





   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Hide(Do_Scheduling_Simulation);
   end On_Cancel_Pressed;




end Do_Scheduling_Simulation_Pkg.Callbacks;



