------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automatically check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use Gtk;
with Gtk.Widget;
use Gtk.Widget;
with Gtk.Enums;
use Gtk.Enums;

with Callbacks_Cheddar;
use Callbacks_Cheddar;

with Text_Io;
use Text_Io;


package body Generic_Select_File_Pkg is






   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Ok_Pressed(To_Unbounded_String(Get_Filename(
               Generic_Select_File)));
      Destroy (Generic_Select_File);
   end On_Ok_Pressed;





   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Cancel_Pressed;
      Destroy (Get_Toplevel (Gtk_Widget (Object)));
   end On_Cancel_Pressed;





   procedure Gtk_New (
         Generic_Select_File :    out Generic_Select_File_Access; 
         Title               : in     Unbounded_String            ) is 

   begin
      Generic_Select_File := new Generic_Select_File_Record;
      Generic_Select_File_Pkg.Initialize (Generic_Select_File, Title);
   end Gtk_New;


   procedure Initialize (
         Generic_Select_File : access Generic_Select_File_Record'Class; 
         Title               : in     Unbounded_String                  ) is 

   begin
      Gtk.File_Selection.Initialize (Generic_Select_File, To_String(
            Title));
      Set_Show_File_Op_Buttons (Generic_Select_File, True);
      Set_Border_Width (Generic_Select_File, 10);
      Set_Title (Generic_Select_File, To_String(Title));
      Set_Policy (Generic_Select_File, False, True, False);
      Set_Position (Generic_Select_File, Win_Pos_Center_Always);
      Set_Modal (Generic_Select_File, True);

      Generic_Select_File.Ok_Button := Get_Ok_Button (
         Generic_Select_File);
      Set_Flags (Generic_Select_File.Ok_Button, Can_Default);
      Button_Callback.Connect
         (Generic_Select_File.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));

      Generic_Select_File.Cancel_Button := Get_Cancel_Button (
         Generic_Select_File);
      Set_Flags (Generic_Select_File.Cancel_Button, Can_Default);
      Button_Callback.Connect
         (Generic_Select_File.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (
            On_Cancel_Pressed'access));

   end Initialize;

end Generic_Select_File_Pkg;
