------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Gtk.Gentry;
use Gtk.Gentry;

with Gtk.Button;
use Gtk.Button;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Mmi_Tools;
use Mmi_Tools;
with objects;
use objects;

with Translate;
use Translate;

with Config;
use Config;

with Network_Set;
use Network_Set;

with Networks;
use Networks;

with Project;
use Project;


with Mmi_Tools;
use Mmi_Tools;

with Generic_Select_File_Pkg;


package body Networks_Parameters_Pkg.Callbacks  is



   procedure Read_Sc_File (
         File_Name : in     Unbounded_String ) is 


   begin

      Set_Text(Networks_Parameters.
         Entry_File_Name, To_String(File_Name));

   end Read_Sc_File;

   procedure Dummy is 
   begin
      null;
   end Dummy;

   package Read_Sc_File_Package is new Generic_Select_File_Pkg
      (Read_Sc_File, Dummy);
   use Read_Sc_File_Package;


   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

   begin

      Sys.Networks:=Networks_Parameters.Undo_Networks;

      Destroy(Networks_Parameters);

   end  On_Cancel_Pressed;


   procedure On_Close_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin

      Destroy(Networks_Parameters);

   end On_Close_Pressed;


   procedure On_Add_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Network_Name : Unbounded_String;  
      Network_Type : Unbounded_String;  

   begin

      Network_Name := To_Unbounded_String(Get_Text(Networks_Parameters.
            Networks_Name));

      if Network_Name=""
            then
         Show_Message_Box(Lb_Networks_Name(Current_Language) &
            Lb_Mandatory(Current_Language));
         return;
      end if;

      if not Is_A_Valid_Identifier(Network_Name) then
         Show_Message_Box(Lb_Networks_Name(Current_Language) &
            Lb_Colon & Lb_Invalid_Identifier(Current_Language));
         return;
      end if;

      Network_Type := To_Unbounded_String(Gtk.Gentry.Get_Text(Get_Entry(
               Networks_Parameters.Networks_Type)));

      if Network_Type = "Bounded delay" then
         Network_Type := To_Unbounded_String(Type_Of_Network'Image(
               Bounded_Delay));
      else
         Network_Type := To_Unbounded_String(Type_Of_Network'Image(
               Parametric_Delay));
      end if;

      Add_Network(Sys.Networks, Network_Name, Network_Type);


   exception
      when  Network_Already_Defined =>
         begin
            Show_Message_Box(Lb_Networks_Name(Current_Language) &
               Lb_Already_Defined(Current_Language) );
            return;
         end;
      when Network_Set.Generic_Network_Set.Full_Set =>
         begin
            Show_Message_Box(Lb_Can_Not_Define_More_Networks(
                  Current_Language));
            return;
         end;

   end On_Add_Pressed;





   procedure On_Read_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Network_Type : Unbounded_String;  

   begin

      -- if the combo selection is not parametric
      --
      Network_Type := To_Unbounded_String(Gtk.Gentry.Get_Text(Get_Entry(
               Networks_Parameters.Networks_Type)));

      if (Network_Type /= "Parametric delay")
            then
         Show_Message_Box(Lb_Only_On_Parametric_Network(Current_Language));
         return;
      end if;


      Gtk_New (Read_Sc_File_Package.Generic_Select_File,  Lb_Read(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language));
      Show_All (Read_Sc_File_Package.Generic_Select_File);


   end On_Read_Pressed;

end Networks_Parameters_Pkg.Callbacks;




