------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use Gtk;
with Gtk.Main;
use Gtk.Main;
with Gtkada.Dialogs;
use Gtkada.Dialogs;





with Open_File_Selection_Pkg;
use Open_File_Selection_Pkg;
use Open_File_Selection_Pkg.Open_Select_File_Pkg;
with Save_File_Selection_Pkg;
use Save_File_Selection_Pkg;
use Save_File_Selection_Pkg.Save_Select_File_Pkg;

with Import_Aadl_Selection_Pkg;
use Import_Aadl_Selection_Pkg;
use Import_Aadl_Selection_Pkg.Import_Aadl_File_Pkg;
with Export_Aadl_Selection_Pkg;
use Export_Aadl_Selection_Pkg;
use Export_Aadl_Selection_Pkg.Export_Aadl_File_Pkg;


with Root_Window_Pkg;
use Root_Window_Pkg;
with Root_Window_Pkg.Callbacks;
use Root_Window_Pkg.Callbacks;
with Call_Scheduling_Framework;
use Call_Scheduling_Framework;
with My_Canvas;
use My_Canvas;
with Graph_Window_Pkg;
use Graph_Window_Pkg;

with Config;
use Config;
with Translate;
use Translate;
with Unbounded_Strings;
use Unbounded_Strings;

with Multiprocessor_Services;
use Multiprocessor_Services;
use Multiprocessor_Services.Scheduling_Result_Per_Processor;




package body Project is



   procedure Initialize (
         B     : in out Data_From_Simulation_Table; 
         Value : in     Boolean                     ) is 
   begin
      for I in   Data_From_Simulation_Table'range loop
         B(I):=Value;
      end loop;
   end Initialize;


   procedure Do_New is 
   begin

      if Root_Window /= null then

         if (not Is_Saved)
               then
            if (not Is_New)
                  then
               if Message_Dialog (To_String(
                        Lb_Not_Saved_Project(Current_Language) & Ascii.Lf &
                        Lb_Continue(Current_Language)),
                     Confirmation,
                     Button_Yes or Button_Cancel,
                     Button_Cancel, -- default
                     "", -- help message
                     "") /= Button_Yes then
                  return;
               end if;
            end if;
         end if;
         Clear_Work_Space;

      end if;

      Initialize(Sys);
      Xml_Project_File_Name:=To_Unbounded_String("");
      Xml_Project_Directory:=To_Unbounded_String("");
      Aadl_Project_File_Name:=To_Unbounded_String("");
      Aadl_Project_Directory:=To_Unbounded_String("");
      Xml_Event_Table_File_Name:=To_Unbounded_String(
         "default_event_table.xml");
      Xml_Event_Table_Directory:=To_Unbounded_String(
         "default_event_table_directory");

      Is_Saved:=False;
      Is_New:=True;


      Schedule_All_Processeurs:=False;
      Schedule_And_Display_Event_Table:=False;
      Schedule_And_Export_Event_Table:=False;
      Event_Table_File_Name:=Empty_String;
      Schedule_With_Offsets:=True;
      Schedule_With_Resources:=True;
      Schedule_With_Precedencies:=True;
      Has_Global_Seed  := True;
      Is_Global_Predictable_Seed := False;
      Global_Seed_Value   := 0;

      Initialize(Response_Time_From_Simulation, False);
      Initialize(Blocking_Time_From_Simulation, False);
      Response_Time_From_Simulation(Worst_Case):=True;
      Context_Switch_From_Simulation := True;
      Preemption_From_Simulation     := True;
      Run_Event_Analyzer_From_Simulation :=True;

      Initialize(Generate_Events, True);
      Initialize(Call_Scheduling_Framework.Sched.All);


   end Do_New;


   procedure Do_Open is 
   begin

      if (not Is_Saved)
            then
         if (not Is_New)
               then
            if Message_Dialog (To_String(
                     Lb_Not_Saved_Project(Current_Language) & Ascii.Lf &
                     Lb_Continue(Current_Language)),
                  Confirmation,
                  Button_Yes or Button_Cancel,
                  Button_Cancel, -- default
                  "", -- help message
                  "") /= Button_Yes then
               return;
            end if;
         end if;
      end if;

      Clear_Work_Space;

      Gtk_New (Open_Select_File_Pkg.Generic_Select_File, Lb_Open_Xml(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language));
      Open_File_Initialize;
      Show_All (Open_Select_File_Pkg.Generic_Select_File);

   end  Do_Open;

   procedure Do_Save is 
   begin


      if Graph_Window/=null
            then
         Get_Coordinates(Graph_Window.Buffer, Sys);
      end if;

      if (Xml_Project_File_Name /= To_Unbounded_String(""))
            then
         Systems.Write_To_Xml_File(Sys, Xml_Project_File_Name);

         Is_Saved:=True;
      else
         Gtk_New (Save_Select_File_Pkg.Generic_Select_File, Lb_Save_Xml(
               Current_Language) & To_Unbounded_String(" ") & Lb_File(
               Current_Language));
         Save_File_Initialize;
         Show_All (Save_Select_File_Pkg.Generic_Select_File);
      end if;

   end  Do_Save;

   procedure Do_Aadl_Export is 
   begin
      Gtk_New (Export_Aadl_File_Pkg.Generic_Select_File, Lb_Save(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language));
      Export_Aadl_Initialize;
      Show_All (Export_Aadl_File_Pkg.Generic_Select_File);

   end  Do_Aadl_Export;

   procedure Do_Aadl_Import is 
   begin

      if (not Is_Saved)
            then
         if (not Is_New)
               then
            if Message_Dialog (To_String(
                     Lb_Not_Saved_Project(Current_Language) & Ascii.Lf &
                     Lb_Continue(Current_Language)),
                  Confirmation,
                  Button_Yes or Button_Cancel,
                  Button_Cancel, -- default
                  "", -- help message
                  "") /= Button_Yes then
               return;
            end if;
         end if;
      end if;

      Clear_Work_Space;

      Gtk_New (Import_Aadl_File_Pkg.Generic_Select_File, Lb_Open(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language));
      Import_Aadl_Initialize;
      Show_All (Import_Aadl_File_Pkg.Generic_Select_File);

   end  Do_Aadl_Import;


   procedure Do_Save_As is 
   begin
      Gtk_New (Save_Select_File_Pkg.Generic_Select_File, Lb_Save_As_Xml(
            Current_Language) & To_Unbounded_String(" ") & Lb_File(
            Current_Language));
      Save_File_Initialize;
      Show_All (Save_Select_File_Pkg.Generic_Select_File);
   end  Do_Save_As;



   procedure Do_Exit is 
   begin
      if (not Is_Saved)
            then
         if (not Is_New)
               then
            if Message_Dialog (To_String(
                     Lb_Not_Saved_Project(Current_Language) & Ascii.Lf &
                     Lb_Continue(Current_Language)),
                  Confirmation,
                  Button_Yes or Button_Cancel,
                  Button_Cancel, -- default
                  "", -- help message
                  "") = Button_Yes then
               Gtk.Main.Gtk_Exit (0);
            else
               return;
            end if;
         end if;
      end if;
      Gtk.Main.Gtk_Exit (0);
   end  Do_Exit;


end Project;

