------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Systems;
use Systems;

with Time_Units;
use Time_Units;


package Project is



   -- The working project
   --
   Sys : System;  


   -- File name and directory of the working XML/AADL project file
   --
   Xml_Project_Directory  : Unbounded_String;  
   Xml_Project_File_Name  : Unbounded_String;  
   Aadl_Project_Directory : Unbounded_String;  
   Aadl_Project_File_Name : Unbounded_String;  


   -- File name and directory of the working XML event table file
   --
   Xml_Event_Table_Directory : Unbounded_String;  
   Xml_Event_Table_File_Name : Unbounded_String;  


   -- Is the project modified, should we have to save it ?
   --
   Is_Saved : Boolean;  
   Is_New   : Boolean;  


   ---------------------------------------------
   -- options used tuned by the user
   ---------------------------------------------

   -- Each item of this boolean table store the
   -- user's choose on which event the simulator engine
   -- will generate
   --
   Generate_Events : Time_Unit_Event_Type_Boolean_Table;  

   -- should we do simulations on all processors ?
   --
   Schedule_All_Processeurs : Boolean;  

   -- should we display the event table after simulation ?
   --
   Schedule_And_Display_Event_Table : Boolean;  

   -- should we save the event table after simulation ?
   --
   Schedule_And_Export_Event_Table : Boolean;  
   Event_Table_File_Name           : Unbounded_String;  

   -- should we do simulations according to offset ?
   --
   Schedule_With_Offsets : Boolean;  

   -- should we do simulations according to shared resources ?
   --
   Schedule_With_Resources : Boolean;  

   -- should we do simulations according to task precedencies ?
   --
   Schedule_With_Precedencies : Boolean;  

   -- should we use only one seed to all random values
   -- generated during simulations ?
   --
   Has_Global_Seed : Boolean;  

   -- should we use seed with a predictable value to be able
   -- to do predictable simulation  ?
   --
   Is_Global_Predictable_Seed : Boolean;  

   -- if Has_Global_Seed and Is_Global_Predictable_Seed
   -- are True, Global_Seed_Value store the seed
   --
   Global_Seed_Value : Integer;  




   type Data_From_Simulation is 
         (Worst_Case,   
          Best_Case,    
          Average_Case, 
          Plot_Case); 
   type Data_From_Simulation_Table is array (Data_From_Simulation'range) of Boolean; 

   procedure Initialize (
         B     : in out Data_From_Simulation_Table; 
         Value : in     Boolean                     ); 



   -- should we compute, from a simulation worst/best/average/plot cases
   -- from response time of blocking time
   --
   Response_Time_From_Simulation : Data_From_Simulation_Table;  
   Blocking_Time_From_Simulation : Data_From_Simulation_Table;  


   -- should we compute from simulation these data ?
   --
   Context_Switch_From_Simulation     : Boolean;  
   Preemption_From_Simulation         : Boolean;  
   Run_Event_Analyzer_From_Simulation : Boolean;  


   -- Processor name on which we should compute something
   --
   Last_Selected_Processor : Unbounded_String;  



   -- Sub programs do project management
   --
   procedure Do_New; 
   procedure Do_Open; 
   procedure Do_Save; 
   procedure Do_Save_As; 
   procedure Do_Exit; 
   procedure Do_Aadl_Export; 
   procedure Do_Aadl_Import; 


end Project;


