------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool. 
-- This program provides services to automatically check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------


with Translate;
use Translate;

with Project;
use Project;
with Systems;
use Systems;

with Ada.Io_Exceptions;
use Ada.Io_Exceptions;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Mmi_Tools;
use Mmi_Tools;
with Graph_Window_Pkg;
use Graph_Window_Pkg;
with My_Canvas;
use My_Canvas;

with Config;
use Config;

with Text_Io;
use Text_Io;


with  Gnat.Current_Exception;
use  Gnat.Current_Exception;



package body Save_File_Selection_Pkg is


   procedure Save_File_Ok_Pressed (
         File : in     Unbounded_String ) is 

   begin

      if Graph_Window/=null
            then
         Get_Coordinates(Graph_Window.Buffer, Sys);
      end if;

      Xml_Project_File_Name := File;
      Systems.Write_To_Xml_File(Sys, Xml_Project_File_Name);
      Xml_Project_Directory:=Xml_Project_File_Name;


      Is_Saved:=True;
      Is_New:=False;


   exception
      when Ada.Io_Exceptions.Name_Error =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Name_Error"));
      when Ada.Io_Exceptions.Status_Error =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Status_Error"));
      when Ada.Io_Exceptions.Mode_Error   =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Mode_Error"));
      when Ada.Io_Exceptions.Use_Error     =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Use_Error"));
      when Ada.Io_Exceptions.Device_Error  =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Device_Error"));
      when Ada.Io_Exceptions.End_Error    =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", End_Error"));
      when Ada.Io_Exceptions.Data_Error   =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Data_Error"));
      when Ada.Io_Exceptions.Layout_Error  =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) &
            To_Unbounded_String(", Layout_Error"));
      when others =>
         Show_Message_Box(Lb_Can_Not_Save_File(Current_Language) 
            & " ; Exception name    : " & Exception_Name & " ; "
            & "Exception message : " & Exception_Message);



   end Save_File_Ok_Pressed;



   procedure Save_File_Cancel_Pressed is 
   begin
      null;
   end Save_File_Cancel_Pressed;



   procedure Save_File_Initialize is 

   begin

      if Xml_Project_Directory=To_Unbounded_String("")
            then
         Xml_Project_Directory:=To_Unbounded_String(Get_Filename(
               Generic_Select_File));
      else
         Set_Filename(Generic_Select_File, To_String(
               Xml_Project_Directory));
      end if;


   end Save_File_Initialize;



end Save_File_Selection_Pkg;
