------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------

with  Gnat.Current_Exception;
use  Gnat.Current_Exception;

with Glib;
use Glib;

with Text_Io;
use Text_Io;

with Translate;
use Translate;

with Config;
use Config;

with Unbounded_Strings;
use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;

with Project;
use Project;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Gtk.Gentry;
use Gtk.Gentry;
with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;



with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;


with Address_Space_Set;
use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;

with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;

with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;




package body Update_Address_Spaces_Set_Pkg.Callbacks.Integrity_Checks is



   procedure Check_Integrity_Constraints (
         Update_Address_Spaces_Set : in out Update_Address_Spaces_Set_Access; 
         Error_Message             : in out Unbounded_String;                 
         Ok                        : in out Boolean                           ) is 


      Address_Space_Data_String  : Unbounded_String;  
      Address_Space_Text_String  : Unbounded_String;  
      Address_Space_Heap_String  : Unbounded_String;  
      Address_Space_Stack_String : Unbounded_String;  

      Cpu_Index   : Gint          := 0;  
      A_Processor : Processor_Ptr;  
      Integer_Ok  : Boolean;  


   begin

      Ok:=True;

      Address_Space_Name := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.Name));

      Address_Space_Heap_String := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.Heap_Memory_Size));
      Address_Space_Text_String := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.Text_Memory_Size));
      Address_Space_Data_String := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.Data_Memory_Size));
      Address_Space_Stack_String := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.Stack_Memory_Size));

      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (Update_Address_Spaces_Set.
               Clist_Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors,A_Processor,Processors_Range(
            Cpu_Index));
      Address_Space_Processor :=A_Processor.Name;



      To_Integer(Address_Space_Text_String,Address_Space_Text, Integer_Ok);
      if not Integer_Ok
            then
         if Address_Space_Text_String /= "" then
            Error_Message:=Lb_Text_Memory_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            return;
         end if;
      end if;

      To_Integer(Address_Space_Data_String,Address_Space_Data, Integer_Ok);
      if not Integer_Ok
            then
         if Address_Space_Data_String /= To_Unbounded_String("") then
            Error_Message:=Lb_Data_Memory_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            return;
         end if;
      end if;

      To_Integer(Address_Space_Heap_String,Address_Space_Heap, Integer_Ok);
      if not Integer_Ok
            then
         if Address_Space_Heap_String /= To_Unbounded_String("") then
            Error_Message:=Lb_Heap_Memory_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            return;
         end if;
      end if;


      To_Integer(Address_Space_Stack_String,Address_Space_Stack,
         Integer_Ok);
      if not Integer_Ok
            then
         if Address_Space_Stack_String /= To_Unbounded_String("") then
            Error_Message:=Lb_Stack_Memory_Size(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language);
            Ok:=False;
            return;
         end if;
      end if;



      Check_Address_Space (Sys.Address_Spaces, Integrity_Checks.
         Address_Space_Name,
         Integrity_Checks.Address_Space_Processor,
         Integrity_Checks.Address_Space_Text,
         Integrity_Checks.Address_Space_Stack,
         Integrity_Checks.Address_Space_Data,
         Integrity_Checks.Address_Space_Heap);


   exception
      when others =>
         Ok:=False;
         Error_Message:=To_Unbounded_String(Exception_Message);


   end Check_Integrity_Constraints;




end Update_Address_Spaces_Set_Pkg.Callbacks.Integrity_Checks;

