------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with  Gnat.Current_Exception;
use  Gnat.Current_Exception;


with Glib;
use Glib;

with Mmi_Tools;
use Mmi_Tools;

with Text_Io;
use Text_Io;

with Translate;
use Translate;

with Config;
use Config;

with Unbounded_Strings;
use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;

with Project;
use Project;

with Address_Spaces;
use Address_Spaces;
with Address_Space_Set;
use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Gtk.Gentry;
use Gtk.Gentry;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Address_Spaces;
use Address_Spaces;
with Address_Space_Set;
use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;

with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;


with Update_Address_Spaces_Set_Pkg.Callbacks.Integrity_Checks;



package body Update_Address_Spaces_Set_Pkg.Callbacks is


   procedure Show_Address_Spaces (
         Update_Address_Spaces_Set : access Update_Address_Spaces_Set_Record'Class ) is 

      An_Address_Space    : Address_Space_Ptr;  
      My_Iterator         : Address_Spaces_Iterator;  
      Line                : Gint;  
      Address_Space_Share : Chars_Ptr_Array (1 .. 6);  
      Text                : Unbounded_String;  
      Stack               : Unbounded_String;  
      Data                : Unbounded_String;  
      Heap                : Unbounded_String;  


   begin
      Clear(Update_Address_Spaces_Set.Address_Spaces_List);

      Reset_Iterator(Sys.Address_Spaces, My_Iterator);
      if not Is_Empty(Sys.Address_Spaces) then
         loop

            Current_Element(Sys.Address_Spaces, An_Address_Space,
               My_Iterator);

            Stack := To_Unbounded_String(
               Natural'Image(An_Address_Space.Stack_Memory_Size));
            Data := To_Unbounded_String(
               Natural'Image(An_Address_Space.Data_Memory_Size));
            Heap := To_Unbounded_String(
               Natural'Image(An_Address_Space.Heap_Memory_Size));
            Text := To_Unbounded_String(
               Natural'Image(Natural(An_Address_Space.Text_Memory_Size)));


            -- Create the line
            --
            Address_Space_Share := To_String(An_Address_Space.Name)+
               To_String(An_Address_Space.Cpu_Name)+To_String(Stack)+
               To_String(Data)+To_String(Heap)+To_String(Text);

            -- Display the Line
            --
            Line := Append(Update_Address_Spaces_Set.Address_Spaces_List,
               Address_Space_Share);

            exit when Is_Last_Element(Sys.Address_Spaces, My_Iterator);

            Next_Element(Sys.Address_Spaces, My_Iterator);

         end loop;
      end if;
   end Show_Address_Spaces;


   procedure On_Click_List (
         List : access Gtk_Clist_Record'Class ) is 

      Name        : Unbounded_String;  
      Cpu_Name    : Unbounded_String;  
      Text        : Unbounded_String;  
      Stack       : Unbounded_String;  
      Data        : Unbounded_String;  
      Heap        : Unbounded_String;  
      Row         : Gint;  
      My_Iterator : Processor_Set.Generic_Processor_Set.Iterator;  
      A_Processor : Processor_Ptr;  
      Index       : Strings_Range                                := Strings_Range'First;  

      Current        : Gint;  
      Location_Items : Strings_Table;  

   begin

      Row:=Get_Focus_Row(Update_Address_Spaces_Set.Address_Spaces_List);
      Name := To_Unbounded_String(Get_Text(Update_Address_Spaces_Set.
            Address_Spaces_List,
            Row,0));
      Update_Address_Spaces_Set.Current_Address_Space := Name;
      Cpu_Name      := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.
            Address_Spaces_List,Row,1));
      Stack      := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.
            Address_Spaces_List,Row,2));
      Data        := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.
            Address_Spaces_List,Row,3));
      Heap      := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.
            Address_Spaces_List,Row,4));
      Text        := To_Unbounded_String(Get_Text(
            Update_Address_Spaces_Set.
            Address_Spaces_List,Row,5));

      Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Name, To_String(Name));
      Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Text_Memory_Size,
         To_String(
            Text));
      Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Heap_Memory_Size,
         To_String(
            Heap));
      Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Data_Memory_Size,
         To_String(
            Data));
      Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Stack_Memory_Size,
         To_String(
            Stack));



      -- Rebuild The CPU Option Menu & select the right item
      --
      Initialize(Location_Items);
      Index:=0;
      Reset_Iterator(Sys.Processors, My_Iterator);

      loop
         Current_Element(Sys.Processors, A_Processor, My_Iterator);
         Add(Location_Items,A_Processor.Name);
         if A_Processor.Name =
               Cpu_Name then
            Current:=Gint(Index);
         end if;
         Index:=Index+1;
         exit when Is_Last_Element(Sys.Processors, My_Iterator);
         Next_Element(Sys.Processors, My_Iterator);
      end loop;

      Update_Address_Spaces_Set.Clist_Processor_Menu:= Widget_Slist.
         Null_List;

      Gtk.Option_Menu.Gtk_New(Update_Address_Spaces_Set.Processor_Menu);

      Build_Option_Menu (Update_Address_Spaces_Set.Processor_Menu,
         Update_Address_Spaces_Set.Clist_Processor_Menu,
         Location_Items, Current, null);


      Show_All(Update_Address_Spaces_Set.Processor_Menu);
      Attach_Defaults(Update_Address_Spaces_Set.Address_Space_Table,
         Update_Address_Spaces_Set.
         Processor_Menu,1,2,1,2);


      Show_All(Update_Address_Spaces_Set);

   end On_Click_List;


   procedure On_Close_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin

      Is_New:=False;
      Is_Saved:=False;

      Destroy(Update_Address_Spaces_Set);
   end On_Close_Pressed;



   procedure On_Delete_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      My_Iterator : Address_Space_Set.Generic_Address_Space_Set.Iterator;  

      An_Address_Space : Address_Space_Ptr;  

   begin



      if Update_Address_Spaces_Set.Current_Address_Space = "" then
         Show_Message_Box(Lb_Select_Address_Space_First(Current_Language));
         return;
      end if;

      if Is_Empty(Sys.Address_Spaces) then
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         return;
      end if;


      Reset_Iterator(Sys.Address_Spaces, My_Iterator);
      if not Is_Empty(Sys.Address_Spaces) then
         loop

            Current_Element(Sys.Address_Spaces, An_Address_Space,
               My_Iterator);
            if An_Address_Space.Name = Update_Address_Spaces_Set.
                  Current_Address_Space then


               Delete_Address_Space(Sys.Resources, An_Address_Space.Name);
               Delete_Address_Space(Sys.Buffers, An_Address_Space.Name);
               Delete_Address_Space(Sys.Tasks, An_Address_Space.Name);
               Delete(Sys.Address_Spaces, An_Address_Space);


               -- Clear  entries
               --
               Update_Address_Spaces_Set.Current_Address_Space :=
                  To_Unbounded_String("");
               Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.Name, "");
               Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.
                  Stack_Memory_Size, "");
               Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.
                  Data_Memory_Size, "");
               Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.
                  Heap_Memory_Size, "");
               Gtk.Gentry.Set_Text(Update_Address_Spaces_Set.
                  Text_Memory_Size, "");

               exit;
            end if;

            exit when Is_Last_Element(Sys.Address_Spaces, My_Iterator);
            Next_Element(Sys.Address_Spaces, My_Iterator);

         end loop;
      end if;

      Show_Address_Spaces(Update_Address_Spaces_Set);


   end On_Delete_Pressed;




   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Duplicate(Update_Address_Spaces_Set.Undo_Address_Space,
         Sys.Address_Spaces);
      Destroy(Update_Address_Spaces_Set);
   end On_Cancel_Pressed;


   procedure On_Advanced_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      if Visible_Is_Set(Update_Address_Spaces_Set.Vbox_List) then
         Show_Advanced(Update_Address_Spaces_Set,False);
      else
         Show_Advanced(Update_Address_Spaces_Set,True);
      end if;
   end On_Advanced_Pressed;



   procedure On_Add_Pressed (
         Object : access Gtk_Button_Record'Class ) is 


      Ok            : Boolean          := True;  
      Error_Message : Unbounded_String;  

   begin


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Address_Spaces_Set, Error_Message, Ok);
      if not Ok then
         Show_Message_Box(Error_Message);
         return;
      end if;


      -- Add the new item in the set
      --
      Add_Address_Space (Sys.Address_Spaces, Integrity_Checks.
         Address_Space_Name,
         Integrity_Checks.Address_Space_Processor,
         Natural(Integrity_Checks.Address_Space_Text),
         Natural(Integrity_Checks.Address_Space_Stack),
         Natural(Integrity_Checks.Address_Space_Data),
         Natural(Integrity_Checks.Address_Space_Heap));

      Update_Address_Spaces_Set.Current_Address_Space :=
         Integrity_Checks.Address_Space_Name;

      Show_Address_Spaces(Update_Address_Spaces_Set);


   exception
      when Address_Space_Set.Invalid_Parameter =>
         begin
            Show_Message_Box(Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box(Exception_Name & Exception_Message);
            return;
         end;

   end On_Add_Pressed;



   procedure On_Modify_Pressed (
         Object : access Gtk_Button_Record'Class ) is 


      An_Address_Space : Address_Space_Ptr;  
      My_Iterator      : Address_Space_Set.Generic_Address_Space_Set.Iterator;  

      Ok            : Boolean          := True;  
      Error_Message : Unbounded_String;  


   begin


      if Update_Address_Spaces_Set.Current_Address_Space = "" then
         Show_Message_Box(Lb_Select_Address_Space_First(Current_Language));
         return;
      end if;

      if Is_Empty(Sys.Address_Spaces) then
         Show_Message_Box(Lb_Can_Not_Modify(Current_Language));
         return;
      end if;


      Integrity_Checks.Check_Integrity_Constraints (
         Update_Address_Spaces_Set, Error_Message, Ok);
      if not Ok then
         Show_Message_Box(Error_Message);
         return;
      end if;



      --
      -- Update set
      --

      Reset_Iterator(Sys.Address_Spaces, My_Iterator);
      loop

         Current_Element(Sys.Address_Spaces, An_Address_Space, My_Iterator);
         if An_Address_Space.Name = Update_Address_Spaces_Set.
               Current_Address_Space then

            Delete(Sys.Address_Spaces, An_Address_Space);

            Add_Address_Space (Sys.Address_Spaces, Integrity_Checks.
               Address_Space_Name,
               Integrity_Checks.Address_Space_Processor,
               Natural(Integrity_Checks.Address_Space_Text),
               Natural(Integrity_Checks.Address_Space_Stack),
               Natural(Integrity_Checks.Address_Space_Data),
               Natural(Integrity_Checks.Address_Space_Heap));


            Update_Address_Spaces_Set.Current_Address_Space :=
               Integrity_Checks.Address_Space_Name;
            exit;
         end if;


         exit when Is_Last_Element(Sys.Address_Spaces, My_Iterator);
         Next_Element(Sys.Address_Spaces, My_Iterator);

      end loop;

      Show_Address_Spaces(Update_Address_Spaces_Set);


   exception
      when Address_Space_Set.Invalid_Parameter =>
         begin
            Show_Message_Box(Exception_Message);
            return;
         end;
      when others =>
         begin
            Show_Message_Box(Exception_Name & Exception_Message);
            return;
         end;



   end On_Modify_Pressed;




end Update_Address_Spaces_Set_Pkg.Callbacks;
