------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a free real time scheduling tool.
-- This program provides services to automatically check temporal constraints
-- of real time tasks.
--
-- Copyright (C) 2002-2007   Frank Singhoff, Jerome Legrand
-- Cheddar is developed by the LISYC Team, University of Brest
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;

with Text_Io;
use Text_Io;

with Translate;
use Translate;

with Config;
use Config;

with Project;
use Project;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Address_Space_Set;
use Address_Space_Set;
use Address_Space_Set.Generic_Address_Space_Set;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Gtk.Gentry;
use Gtk.Gentry;
with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;

with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Buffer_Set;
use Buffer_Set;
use Buffer_Set.Generic_Buffer_Set;
with Resource_Set;
use Resource_Set;
use Resource_Set.Generic_Resource_Set;

with Scheduler;
use Scheduler;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;
use Unbounded_Strings.Strings_Table_Package;

with Networks;
use Networks;
with Network_Set;
use Network_Set;
use Network_Set.Generic_Network_Set;

with  Gnat.Current_Exception;
use  Gnat.Current_Exception;



package body Update_Processors_Set_Pkg.Callbacks.Integrity_Checks is



   procedure Check_Integrity_Constraints (
         Update_Processors_Set : in out Update_Processors_Set_Access; 
         Error_Message         : in out Unbounded_String;             
         Ok                    : in out Boolean                       ) is 



      Sched          : Gint;  
      Net            : Gint;  
      A_Network      : Network_Ptr;  
      Quantum_String : Unbounded_String;  
      Integer_Ok     : Boolean;  


   begin

      Ok:=True;


      Processor_Name:=To_Unbounded_String(Get_Text(
            Update_Processors_Set.Entry_Processor_Name));


      Sched:= Gint'Val (10 -
         Gint(Selected_Button (Update_Processors_Set.Clist_Scheduler_Menu)));

      Net:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Networks)) - Gint(
            Selected_Button (Update_Processors_Set.Clist_Network_Menu)));


      Processor_File_Name :=To_Unbounded_String(Get_Text(
            Update_Processors_Set.
            Entry_File_Name));


      if (Sched=0)
            then
         Processor_Scheduler := Earliest_Deadline_First_Protocol;
      end if;
      if (Sched=1)
            then
         Processor_Scheduler := Least_Laxity_First_Protocol;
      end if;
      if (Sched=2)
            then
         Processor_Scheduler := Rate_Monotonic_Protocol;
      end if;
      if (Sched=3)
            then
         Processor_Scheduler := Deadline_Monotonic_Protocol;
      end if;
      if (Sched=4)
            then
         Processor_Scheduler := Posix_1003_Highest_Priority_First_Protocol;
      end if;
      if (Sched=5)
            then
         Processor_Scheduler := Time_Sharing_Based_On_Wait_Time_Protocol;
      end if;
      if (Sched=6)
            then
         Processor_Scheduler := Round_Robin_Protocol;
      end if;
      if (Sched=7)
            then
         Processor_Scheduler := Maximum_Urgency_First_Protocol;
      end if;
      if (Sched=8)
            then
         Processor_Scheduler := D_Over_Protocol;
      end if;
      if (Sched=9)
            then
         Processor_Scheduler := Time_Sharing_Based_On_Cpu_Usage_Protocol;
      end if;
      if (Sched=10)
            then
         Processor_Scheduler := Parametric_Protocol;
      end if;




      if (Net = 0)
            then
         Processor_Network :=  To_Unbounded_String("No_Network");
      else
         Get_Element_Number(Sys.Networks,A_Network,
            Network_Range(
               Net-1));
         Processor_Network:=A_Network.Name;
      end if;



      Quantum_String:=To_Unbounded_String(
         Get_Text(
            Update_Processors_Set.
            Entry_Quantum));
      if(Quantum_String="")
            then
         Processor_Quantum:=0;
      else
         To_Integer(Quantum_String, Processor_Quantum, Integer_Ok);
         if not Integer_Ok
               then
            Error_Message:="Quantum" &
               Lb_Must_Be_Numeric(
               Current_Language);
            Ok:=False;
            return;
         end if;
      end if;

      if Get_Active(Update_Processors_Set.
            Entry_Option)
            then
         Processor_Preemptive:=Preemptive;
      else
         Processor_Preemptive:=Not_Preemptive;
      end if;


      Check_Processor(Sys.Processors,
         Integrity_Checks.Processor_Name,
         Integrity_Checks.Processor_Preemptive,
         Integrity_Checks.Processor_Quantum,
         Integrity_Checks.Processor_File_Name,
         Integrity_Checks.Processor_Scheduler,
         Integrity_Checks.Processor_Network);




   exception
      when others =>
         Ok:=False;
         Error_Message:=To_Unbounded_String(Exception_Message);



   end Check_Integrity_Constraints;




end Update_Processors_Set_Pkg.Callbacks.Integrity_Checks;


